#include "stdafx.h"
#include <vector>
#include <afxwin.h>
#include <afxmt.h>
#include "NKFWrapper.h"
//#include "Log.h"

using namespace std;

std::auto_ptr<CMutex> NKFWrapper::m_pMutex = auto_ptr<CMutex>(new CMutex(FALSE, "NKFWrapper"));

NKFWrapper::NKFWrapper() : m_hModule(NULL), SetNkfOption(NULL), NkfConvertSafe(NULL)
{
#ifdef _AFXDLL
	m_hModule = AfxLoadLibrary("nkf32.dll");
#else
	m_hModule = LoadLibrary("nkf32.dll");
#endif
	if (!m_hModule){
		//Log::Add("NKFWrapper::NKFWrapper() : nkf.dll̃[hɎs܂");
		return;
	}

	SetNkfOption = (FuncPtrSetNkfOption)GetProcAddress(m_hModule, "SetNkfOption");
	NkfConvertSafe = (FuncPtrNkfConvertSafe)GetProcAddress(m_hModule, "NkfConvertSafe");
}

NKFWrapper::~NKFWrapper()
{
	if (m_hModule){
#ifdef _AFXDLL
		AfxFreeLibrary(m_hModule);
#else
		FreeLibrary(m_hModule);
#endif
		m_hModule = NULL;
	}
	SetNkfOption = NULL;
	NkfConvertSafe = NULL;
}

bool NKFWrapper::SetOption(const std::string& option)
{
	if (!SetNkfOption){
		//Log::Add("NKFWrapper::NKFWrapper() : nkf.dllǂݍ܂Ă܂");
		return false;
	}

	m_pMutex->Lock();

	if (SetNkfOption((LPCSTR)option.c_str())){
		return false;
	}

	m_pMutex->Unlock();

	return true;
}

bool NKFWrapper::Convert(std::string& s)
{
	if (!NkfConvertSafe){
		//Log::Add("NKFWrapper::NKFWrapper() : nkf.dllǂݍ܂Ă܂");
		return false;
	}

	m_pMutex->Lock();

	vector<char> inbuf(s.begin(), s.end());
	inbuf.push_back('\0');
	vector<char> outbuf(s.size() * 2 + 1);
	DWORD outsize;
	int len = strlen(&inbuf[0]);
	if (!NkfConvertSafe((LPSTR)&outbuf[0], (DWORD)outbuf.size(), (LPDWORD)&outsize, (LPCSTR)&inbuf[0], (DWORD)strlen(&inbuf[0]))){
		return false;
	}

	s = &outbuf[0];

	m_pMutex->Unlock();
	return true;
}
