#include "stdafx.h"
#include "SQLite.h"

using namespace std;

SQLite::SQLite() : m_hDll(NULL), m_pDataBase(NULL)
{
	LoadDll();
}

SQLite::SQLite(const std::string& data_base_file_path) : m_hDll(NULL), m_pDataBase(NULL)
{
	LoadDll();
	Open(data_base_file_path);
}

SQLite::~SQLite()
{
	Close();
	FreeDll();
}

int SQLite::Open(const std::string& data_base_file_path)
{
	char buffer[1024];
	strcpy_s(buffer, sizeof(buffer), data_base_file_path.c_str());
	return sqlite3_open(buffer, &m_pDataBase);
}

int SQLite::GetTable(const std::string& sql, char **&table, int &row, int &column)
{
	char *buffer = new char[sql.size() + 1];
	strcpy_s(buffer, sql.size() + 1, sql.c_str());

	char *err = NULL;

	int res = sqlite3_get_table(m_pDataBase, buffer, &table, &row, &column, &err);

	delete buffer;
	buffer = NULL;

	if (err){
		m_strError = err;
		sqlite3_free(err);
		err = NULL;
	}

	return res;
}

int SQLite::FreeTable(char **&table)
{
	int res = sqlite3_free_table(table);
	table = NULL;
	return res;
}

int SQLite::Exec(const std::string& sql, sqlite3_callback callback, void* param)
{
	char *buf = NULL;
	int res = sqlite3_exec(m_pDataBase, sql.c_str(), callback, param, &buf);
	if (buf){
		m_strError = buf;
		sqlite3_free(buf);
		buf = NULL;
	}
	return res;
}

int SQLite::Close()
{
	return sqlite3_close(m_pDataBase);
	m_pDataBase = NULL;
}

const std::string& SQLite::GetError()const
{
	return m_strError;
}

void SQLite::LoadDll()
{
	// sqlite3.dll[h
	m_hDll = LoadLibrary("sqlite3");

	if (m_hDll == NULL) {
		return;
	}

	// sqlite3֐̃AhX擾
	sqlite3_open = (pOpen)GetProcAddress(m_hDll, "sqlite3_open");
	sqlite3_exec = (pExec)GetProcAddress(m_hDll, "sqlite3_exec");
	sqlite3_get_table = (pGetTable)GetProcAddress(m_hDll, "sqlite3_get_table");
	sqlite3_free_table = (pFreeTable)GetProcAddress(m_hDll, "sqlite3_free_table");
	sqlite3_close = (pClose)GetProcAddress(m_hDll, "sqlite3_close");
	sqlite3_free = (pFree)GetProcAddress(m_hDll, "sqlite3_free");
}

void SQLite::FreeDll()
{
	// sqlite3.dll
	FreeLibrary(m_hDll);

	sqlite3_open = NULL;
	sqlite3_exec = NULL;
	sqlite3_close = NULL;
	sqlite3_free = NULL;
	sqlite3_get_table = NULL;
	sqlite3_free_table = NULL;
}
