#include "stdafx.h"
#include <algorithm>
#include <fstream>
#include <sstream>
#include "SettingFile.h"
#include "Utility.h"

using namespace std;

const int BUFFER_SIZE_IO = 4096;
const int BUFFER_SIZE_READ = 64 * 1024;

bool SettingFile::Load(const std::string& file_path)
{
	FILE *file = NULL;
	if (fopen_s(&file, file_path.c_str(), "rS")){
		return false;
	}
	setvbuf(file, NULL, _IOFBF, 4096);

	char buffer[BUFFER_SIZE_READ + 1];

	string last_section;
	while (fgets(buffer, BUFFER_SIZE_READ, file)){
		int len = (int)strlen(buffer);
		buffer[--len] = '\0';
		if (buffer[len - 1] == '\r'){
			buffer[--len] = '\0';
		}

		if (buffer[0] == '\0'){
			continue;
		}

		if (buffer[0] == '/' && buffer[1] == '/'){
			continue;
		}

		if (buffer[0] == '['){
			buffer[--len] = '\0';
			last_section = &buffer[1];
		} else {
			char *ptr = strchr(buffer, '=');
			*ptr = '\0';
			Add(last_section, buffer, ptr + 1);
		}
	}

	fclose(file);
	file = NULL;

	Rebuild();
	return true;
}

bool SettingFile::Save(const std::string& file_path)const
{
	//FILE *file = NULL;
	//if (fopen_s(&file, file_path.c_str(), "w")){
	//	return false;
	//}

	//for (map<string, map<string, string> >::const_iterator it_section = m_Table.begin(); it_section != m_Table.end(); ++it_section){
	//	fprintf(file, "[%s]\n", it_section->first.c_str());
	//	for (map<string, string>::const_iterator it_key = it_section->second.begin(); it_key != it_section->second.end(); ++it_key){
	//		fprintf(file, "%s=%s\n", it_key->first.c_str(), it_key->second.c_str());
	//	}
	//}

	//fclose(file);
	//file = NULL;

	ofstream ofs(file_path.c_str());
	if (!ofs.is_open()){
		return false;
	}
	for (map<string, map<string, string> >::const_iterator it_section = m_Table.begin(); it_section != m_Table.end(); ++it_section){
		ofs << "[" << it_section->first << "]" << endl;
		for (map<string, string>::const_iterator it_key = it_section->second.begin(); it_key != it_section->second.end(); ++it_key){
			ofs << it_key->first << "=" << it_key->second << endl;
		}
	}

	return true;
}

const std::vector<std::string> SettingFile::GetKeys(const std::string& section)const
{
	vector<string> res;
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return res;
	}
	res.reserve(it_section->second.size());
	for (map<string, string>::const_iterator it_key = it_section->second.begin(); it_key != it_section->second.end(); ++it_key){
		res.push_back(it_key->first);
	}
	return res;
}

const string strEmpty("");

const std::string& SettingFile::Get(const std::string& section, const std::string& key)const
{
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return strEmpty;
	}
	map<string, string>::const_iterator it_key = it_section->second.find(key);
	if (it_key == it_section->second.end()){
		return strEmpty;
	}
	return it_key->second;
}

void SettingFile::Add(const std::string& section, const std::string& key, const std::string& value)
{
	m_Table[section][key] = value;
}

long long SettingFile::GetInt(const std::string& section, const std::string& key)const
{
	const std::string str = Get(section, key);
	if (str.empty()){
		return 0;
	}
	return Utility::ToInt(str);
}

void SettingFile::AddInt(const std::string& section, const std::string& key, long long value)
{
	Add(section, key, Utility::ToString(value, 0));
}

bool SettingFile::IsInSection(const std::string& section)const
{
	return m_Table.find(section) != m_Table.end();
}

bool SettingFile::IsInKey(const std::string& section, const std::string& key)const
{
	map<string, map<string, string> >::const_iterator it_section = m_Table.find(section);
	if (it_section == m_Table.end()){
		return false;
	}
	return it_section->second.find(key) != it_section->second.end();
}

void SettingFile::Rebuild()
{
	m_vecSection.clear();
	m_vecSection.reserve(m_Table.size());
	for (std::map<std::string, std::map<std::string, std::string> >::iterator it = m_Table.begin(); it != m_Table.end(); ++it){
		m_vecSection.push_back(it->first);
	}
}
