#include "stdafx.h"
#include <atlutil.h>
#include <afxmt.h>
#include "ThreadPool.h"
#include "ThreadPoolWorkerBase.h"

using namespace std;

/*****************************************************************************
 * Xbhv[̎
 *****************************************************************************/

ThreadPool::ThreadPool(int pool_size)
{
	m_pThreadPool = auto_ptr<CThreadPool<Worker> >(new CThreadPool<Worker>());
	m_pThreadPool->Initialize(this, pool_size);
	m_hSignal = CreateEvent(NULL, FALSE, TRUE, NULL);
	m_iThreadCount = 0;
	m_pCriticalSection = auto_ptr<CCriticalSection>(new CCriticalSection());
}

ThreadPool::~ThreadPool()
{
	WaitForSingleObject(m_hSignal, INFINITE);
	CloseHandle(m_hSignal);
	m_hSignal = NULL;
}

bool ThreadPool::Add(ThreadPoolWorkerBase *worker)
{
	IncCount();
	m_pThreadPool->QueueRequest(worker);
	return true;
}

void ThreadPool::WaitFinish()
{
	WaitForSingleObject(m_hSignal, INFINITE);
}

void ThreadPool::IncCount()
{
	m_pCriticalSection->Lock();
	if (m_iThreadCount++ == 0){
		ResetEvent(m_hSignal);
	}
	//Log::Add("Inc : " + Utility::ToString(m_iThreadCount));
	m_pCriticalSection->Unlock();
}

void ThreadPool::DecCount()
{
	m_pCriticalSection->Lock();
	int count;
	m_pThreadPool->GetSize(&count);
	--m_iThreadCount;
	//Log::Add("Dec : " + Utility::ToString(m_iThreadCount));
	m_pCriticalSection->Unlock();
	if (m_iThreadCount == 0){
		SetEvent(m_hSignal);
	}
}

int ThreadPool::GetThreadCount()
{
	m_pCriticalSection->Lock();
	int res = m_iThreadCount;
	m_pCriticalSection->Unlock();
	return res;
}

/*****************************************************************************
 * [J[NX̎
 *****************************************************************************/
BOOL ThreadPool::Worker::Initialize(void* pvWorkerParam)
{
	return TRUE;
}

void ThreadPool::Worker::Execute(RequestType request, void* pvWorkerParam, OVERLAPPED* pOverlapped)
{
	ThreadPoolWorkerBase* worker = request;
	request = NULL;

	worker->Run();
	delete worker;
	worker = NULL;

	ThreadPool* thread_pool = (ThreadPool*)pvWorkerParam;
	thread_pool->DecCount();
}

void ThreadPool::Worker::Terminate(void* pvWorkerParam)
{
}
