#include "stdafx.h"
#include "Utility.h"
#include <cstdio>
#include <cstdlib>
#include <sstream>
#include <fstream>
#include <memory>
#include <boost/crc.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>
#include <afx.h>
#include "Log.h"
#include "NKFWrapper.h"

using namespace std;

string strCurrentFolder;
vector<string> vecFindFile;
char cByteTable[256];
const int BUFFER_SIZE_PATH = 1024;
const int BUFFER_SIZE_IO = 4096;
SYSTEM_INFO m_SystemInfo;

#ifdef USE_NKF
auto_ptr<NKFWrapper> pNKFWrapper;
#endif

void Utility::Initialize(const std::string& log_file_name)
{
	srand((unsigned int)time(NULL));
	char buffer[MAX_PATH];
	::GetModuleFileName(NULL, buffer, MAX_PATH);
	strCurrentFolder = Utility::SplitFileFolder(buffer);
//	string current_folder = buffer;
//	current_folder.erase(current_folder.find_last_of('\\') + 1);
//	strCurrentFolder = current_folder;

	for (int i = 0; i < 256; ++i){
		cByteTable[i] = -1;
	}
	for (char c = '0'; c <= '9'; ++c){
		cByteTable[(int)c] = c - '0';
	}
	for (char c = 'a'; c <= 'f'; ++c){
		cByteTable[(int)c] = c - 'a' + 10;
	}
	for (char c = 'A'; c <= 'F'; ++c){
		cByteTable[(int)c] = c - 'A' + 10;
	}
	setlocale(LC_ALL, "Japanese");
	::SetCurrentDirectory(strCurrentFolder.c_str());

	::GetSystemInfo(&m_SystemInfo);

	Log::Start(log_file_name);

#ifdef USE_NKF
	pNKFWrapper = auto_ptr<NKFWrapper>(new NKFWrapper());
#endif
}

void FindFileEx(const std::string& folder, const std::string& file_name, bool sub_folder)
{
	if (sub_folder){
		string wild_card = folder;
		if (wild_card[wild_card.size() - 1] != '\\'){
			wild_card += "\\";
		}
		wild_card += "*.*";
		CFileFind finder_folder;
		BOOL working = finder_folder.FindFile(wild_card.c_str());
		while (working){
			working = finder_folder.FindNextFile();
			if (finder_folder.IsDots()){
				continue;
			}
			if (finder_folder.IsDirectory()){
				string next = (LPCTSTR)finder_folder.GetFilePath();
				FindFileEx(next, file_name, sub_folder);
			}
		}
		finder_folder.Close();
	}

	string wild_card;
	wild_card = folder;
	if (wild_card[wild_card.size() - 1] != '\\'){
		wild_card += "\\";
	}
	wild_card += file_name;
	CFileFind finder_file;
	BOOL working = finder_file.FindFile(wild_card.c_str());
	while (working){
		working = finder_file.FindNextFile();
		if (finder_file.IsDots()){
			continue;
		}
		if (finder_file.IsDirectory()){
			continue;
		}
		vecFindFile.push_back((LPCTSTR)finder_file.GetFilePath());

	}
	finder_file.Close();
}

const vector<string>& Utility::FindFile(const std::string& folder, const std::string& file_name, bool sub_folder)
{
	vecFindFile.clear();
	FindFileEx(folder, file_name, sub_folder);
	return vecFindFile;
}

const string& Utility::GetProgramFolder()
{
	return strCurrentFolder;
}

const std::string Utility::GetCurrentFolder()
{
	char buffer[BUFFER_SIZE_PATH + 1];
	ZeroMemory(buffer, sizeof(buffer));
	::GetCurrentDirectory(BUFFER_SIZE_PATH, buffer);
	return string(buffer) + "\\";
}

int Utility::GetClusterSize(const std::string& drive)
{
	DWORD secter_per_cluster, bytes_per_sector, number_of_free_clusters, total_number_of_cluseters;
	::GetDiskFreeSpace(((drive.empty()) ? NULL : drive.c_str()), &secter_per_cluster, &bytes_per_sector, &number_of_free_clusters, &total_number_of_cluseters);
	return (int)(secter_per_cluster * bytes_per_sector);
}

bool Utility::IsExist(const std::string& file_path)
{
	CFileStatus status;
	return CFile::GetStatus(file_path.c_str(), status) == TRUE;
}

void Utility::MessageBox(const std::string& message)
{
	AfxMessageBox(message.c_str());
}

//GetFolderNameFromWindow R[obN֐
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lParam,LPARAM lpData)
{
    if(uMsg==BFFM_INITIALIZED){
        SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)TRUE,lpData);
    }
    return 0;
}

const std::string Utility::GetFolderNameFromWindow(HWND hwnd, const std::string& folder_path)
{
	LPMALLOC g_pMalloc;

	// IMallocC^tF[X̎擾
	SHGetMalloc(&g_pMalloc);

	BROWSEINFO  bi;
	ZeroMemory(&bi, sizeof(BROWSEINFO));
	char buffer[MAX_PATH];
	bi.hwndOwner        = hwnd;
	bi.pidlRoot         = NULL;
	bi.pszDisplayName   = buffer;
	bi.lpszTitle        = "tH_IĂ";
	bi.ulFlags          = BIF_RETURNONLYFSDIRS | BIF_USENEWUI;
	bi.lpfn             = &BrowseCallbackProc;
	bi.lParam           = (LPARAM)folder_path.c_str();
	bi.iImage           = 0;
	//tH_̎Qƃ_CAO\
	string res;
	ITEMIDLIST  *idl = SHBrowseForFolder(&bi);
	if(idl != NULL)
	{
		SHGetPathFromIDList(idl, buffer);
		g_pMalloc->Release();
		res = buffer;
	}
	CoTaskMemFree(idl);
	return res;
}

const string Utility::GetCRC(const std::string& file_path)
{
	int file_size = (int)Utility::GetFileSize(file_path);
	if (file_size < 0){
		return "FFFFFFFF";
	}

	FILE *file = NULL;
	if (fopen_s(&file, file_path.c_str(), "rbS")){
		return "FFFFFFFF";
	}

	vector<char> buffer;
	buffer.reserve(file_size);

	char read_buffer[BUFFER_SIZE_IO];
	int read_size;
	while ((read_size = (int)fread(read_buffer, 1, BUFFER_SIZE_IO, file)) != 0){
		buffer.insert(buffer.end(), read_buffer, read_buffer + read_size);
	}

	fclose(file);
	file = NULL;

	return GetCRCBuffer(&buffer[0], file_size);
}

const string Utility::GetCRCBuffer(const char* ptr, unsigned int size)
{
	boost::crc_32_type boost_crc;
	boost_crc.process_bytes(ptr, size);
	unsigned int crc = boost_crc.checksum();
	return ToHex(crc, 8);
}

const string Utility::ToLower(const string& in)
{
	vector<char> buffer;
	buffer.reserve(in.size() + 1);
	for (string::const_iterator it = in.begin(); it != in.end(); ++it){
		buffer.push_back((char)tolower(*it));
	}
	buffer.push_back('\0');
	return &buffer[0];
}

const string Utility::ToUpper(const string& in)
{
	vector<char> buffer;
	buffer.reserve(in.size() + 1);
	for (string::const_iterator it = in.begin(); it != in.end(); ++it){
		buffer.push_back((char)toupper(*it));
	}
	buffer.push_back('\0');
	return &buffer[0];
}

const std::string Utility::GenerateBackupFileName(const std::string& file_path)
{
	for (int i = 0; i < 1000; ++i){
		string new_file_path = file_path + "." + Utility::ToString(i, 3);
		if (!Utility::IsExist(new_file_path)){
			return new_file_path;
		}
	}
	return "";
}

bool Utility::IsFile(const string& file_path)
{
	CFileStatus status;
	if (!CFile::GetStatus(file_path.c_str(), status)){
		return false;
	}
	return (!(status.m_attribute & CFile::directory)) ? true : false;
}

bool Utility::IsFolder(const string& file_path)
{
	CFileStatus status;
	if (!CFile::GetStatus(file_path.c_str(), status)){
		return false;
	}
	return (status.m_attribute & CFile::directory) ? true : false;
}

vector<string> vecDeleteObject;
void DeleteFolderRef(const std::string& folder_path)
{
	CFileFind finder;
	BOOL working = finder.FindFile((folder_path + "\\*.*").c_str());
	while (working){
		working = finder.FindNextFile();
		if (finder.IsDots()){
			continue;
		}
		if (finder.IsDirectory()){
			DeleteFolderRef((LPCTSTR)finder.GetFilePath());
			continue;
		}
		vecDeleteObject.push_back((LPCTSTR)finder.GetFilePath());
	}
	finder.Close();
	vecDeleteObject.push_back(folder_path);
}

void Utility::DeleteFolder(const std::string& folder_path)
{
	vecDeleteObject.clear();
	DeleteFolderRef(folder_path);
	for (vector<string>::iterator it = vecDeleteObject.begin(); it != vecDeleteObject.end(); ++it){
		if (Utility::IsFile(*it)){
			if (!::DeleteFile(it->c_str())){
				Log::Add("t@C폜ł܂ł " + *it);
				Log::LastError(GetLastError());
			}
		} else if (Utility::IsFolder(*it)){
			if (!::RemoveDirectory(it->c_str())){
				Log::Add("tH_폜ł܂ł " + *it);
				Log::LastError(GetLastError());
			}
		}
	}
	vecDeleteObject.clear();
}

const string Utility::GetRootFolder(const std::string& folder_path)
{
	wstring work = StringToWString(folder_path);
	if (work.find_last_of(L"\\") == 0){
		return folder_path;
	}
	work.erase(work.find_last_of(L"\\"));
	return WStringToString(work);
}

const vector<char> vecEmpty;

const vector<char> Utility::ConvertByteString(const std::string& str)
{
	if (str.size() % 2 == 1){
		return vecEmpty;
	}
	for (string::const_iterator it = str.begin(); it != str.end(); ++it){
		if (cByteTable[(int)*it] < 0){
			return vecEmpty;
		}
	}
	size_t index = 0;
	vector<char> res;
	res.reserve(str.size() / 2);
	while (index != str.size()){
		res.push_back((cByteTable[str[index]] << 4) + cByteTable[str[index + 1]]);
		index += 2;
	}
	return res;
}

long long Utility::GetFileSize(const std::string& file_path)
{
	if (!Utility::IsFile(file_path)){
		return -1;
	}
	return (long long)CFile(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone).GetLength();
}

// l<=>ϊn
const int BUFFER_SIZE_FORMAT = 256;

const std::string Utility::ToHex(long long number, int digit)
{
	string format = "%";
	if (digit != 0){
		format += "0";
		format += ToString(digit);
	}
	format += "x";

	char buffer[BUFFER_SIZE_FORMAT + 1];
	sprintf_s(buffer, BUFFER_SIZE_FORMAT, format.c_str(), number);
	return buffer;
}

long long Utility::ToInt(const std::string& str)
{
	for (string::const_iterator it = str.begin(); it != str.end(); ++it){
		if (!isdigit(*it)){
			return -1;
		}
	}
	return boost::lexical_cast<long long>(str);
}

const std::string Utility::ToString(long long number, int digit)
{
	string format = "%";
	if (digit){
		format += "0";
		format += boost::lexical_cast<string>(digit);
	}
	format += "ld";

	char buffer[BUFFER_SIZE_FORMAT + 1];
	sprintf_s(buffer, BUFFER_SIZE_FORMAT, format.c_str(), number);
	return buffer;
}

long long Utility::HexToInt(const std::string& hex)
{
	if (hex.size() > 16){
		return -1;
	}
	long long res = 0;
	for (string::const_iterator it = hex.begin(); it != hex.end(); ++it){
		if (cByteTable[*it] < -1){
			return -1;
		}
		res = (res << 4) | (long long)cByteTable[*it];
	}
	return res;
}

const std::string Utility::ToHalf(const std::string& in)
{
	const int buffer_size = (int)(in.size() * 2);
	vector<char> buffer(buffer_size);
	LCMapString(MAKELCID(MAKELANGID(LANG_JAPANESE, SUBLANG_DEFAULT), SORT_DEFAULT), LCMAP_HALFWIDTH, in.c_str(), -1, &buffer[0], (int)in.size() * 2);
	return &buffer[0];
}

const std::string Utility::ToFull(const std::string& in)
{
	const int buffer_size = (int)(in.size() * 3);
	vector<char> buffer(buffer_size);
	LCMapString(MAKELCID(MAKELANGID(LANG_JAPANESE, SUBLANG_DEFAULT), SORT_DEFAULT), LCMAP_FULLWIDTH, in.c_str(), -1, &buffer[0], (int)in.size() * 2);
	return &buffer[0];
}

const int BUFFER_SIZE_FILE_NAME = 64 * 1024;

const std::vector<std::string> Utility::GetFileNameFromWindow(bool is_open, bool is_multi, const std::string& extention, const std::string& init_file_name, int flag, const std::string& filter, CWnd* wnd)
{
	vector<TCHAR> buffer;
	BOOL bOpenFileDialog = is_open ? TRUE : FALSE;
	LPCTSTR lpszDefExt = extention.empty() ? NULL : extention.c_str();
	LPCTSTR lpszFileName = init_file_name.empty() ? NULL : init_file_name.c_str();
	DWORD dwFlags = flag;
	if (is_multi){
		buffer.resize(BUFFER_SIZE_FILE_NAME);
		flag |= OFN_ALLOWMULTISELECT;
	}
	LPCTSTR lpszFilter = filter.empty() ? NULL : filter.c_str();
	CWnd* pParentWnd = wnd;
	CFileDialog dialog(bOpenFileDialog, lpszDefExt, lpszFileName, dwFlags, lpszFilter, pParentWnd);
	if (is_multi){
		dialog.GetOFN().lpstrFile = &buffer[0];
		dialog.GetOFN().nMaxFile = BUFFER_SIZE_FILE_NAME;
		dialog.GetOFN().Flags |= OFN_ALLOWMULTISELECT;
	}
	vector<string> res;
	if (dialog.DoModal() == IDOK){
		POSITION pos = dialog.GetStartPosition();
		while (pos != NULL){
			res.push_back((LPCTSTR)dialog.GetNextPathName(pos));
		}
	}
	return res;
}

const std::string Utility::WStringToString(const std::wstring& in)
{
	int size = WideCharToMultiByte(932, 0, in.c_str(), -1, NULL, 0, NULL, NULL) + 1;
	auto_ptr<char> buffer = auto_ptr<char>(new char[size]);
	ZeroMemory(buffer.get(), size * sizeof(char));
	WideCharToMultiByte(CP_ACP, 0, in.c_str(), -1, buffer.get(), size, NULL, NULL);
	return string(buffer.get());
}

const std::wstring Utility::StringToWString(const std::string& in)
{
	int size = MultiByteToWideChar(932, 0, in.c_str(), -1, NULL, 0) + 1;
	auto_ptr<WCHAR> buffer = auto_ptr<WCHAR>(new WCHAR[size]);
	ZeroMemory(buffer.get(), size * sizeof(WCHAR));
	MultiByteToWideChar(CP_ACP, 0, in.c_str(), -1, buffer.get(), size);
	return wstring(buffer.get());
}

std::string& Utility::FilterNonPrintableChar(std::string& in)
{
	string temp;
	for (string::iterator it = in.begin(); it != in.end(); ++it){
		if (isprint(*it)){
			temp.push_back(*it);
		}
	}
	swap(temp, in);
	return in;
}

const std::string Utility::SplitFileName(const std::string& file_path)
{
	char buffer_fname[_MAX_FNAME + 1];
	char buffer_ext[_MAX_EXT + 1];
	memset(buffer_fname, 0, sizeof(buffer_fname));
	memset(buffer_ext, 0, sizeof(buffer_ext));
	_splitpath_s(file_path.c_str(), NULL, 0, NULL, 0, buffer_fname, _MAX_FNAME, buffer_ext, _MAX_EXT);
	return string(buffer_fname) + buffer_ext;
}

const std::string Utility::SplitFileTitle(const std::string& file_path)
{
	char buffer_fname[_MAX_FNAME + 1];
	memset(buffer_fname, 0, sizeof(buffer_fname));
	_splitpath_s(file_path.c_str(), NULL, 0, NULL, 0, buffer_fname, _MAX_FNAME, NULL, 0);
	return buffer_fname;
}

const std::string Utility::SplitFileExtention(const std::string& file_path)
{
	char buffer_ext[_MAX_EXT + 1];
	memset(buffer_ext, 0, sizeof(buffer_ext));
	_splitpath_s(file_path.c_str(), NULL, 0, NULL, 0, NULL, 0, buffer_ext, _MAX_EXT);
	string res = ToLower(buffer_ext);
	if (!res.empty()){
		res.erase(0, 1);
	}
	return res;
}

const std::string Utility::SplitFileFolder(const std::string& file_path)
{
	char buffer_drive[_MAX_DRIVE + 1];
	char buffer_dir[_MAX_DIR + 1];
	memset(buffer_drive, 0, sizeof(buffer_drive));
	memset(buffer_dir, 0, sizeof(buffer_dir));
	_splitpath_s(file_path.c_str(), buffer_drive, _MAX_DRIVE, buffer_dir, _MAX_DIR, NULL, 0, NULL, 0);
	return string(buffer_drive) + buffer_dir;
}

int Utility::Square2(int i)
{
	int bit= 0;
	--i;
	while (i){
		i /= 2;
		++bit;
	}
	return (1 << bit);
}

int Utility::GetNumberOfProcessors()
{
	return (int)m_SystemInfo.dwNumberOfProcessors;
}
