#include "stdafx.h"
#include <sstream>
#include <vector>
#include <string>
#include <boost/format.hpp>
#include <boost/regex.hpp>

#include "Resource.h"
#include "memccppDlg.h"
#include "Control.h"
#include "SettingFile.h"
#include "Log.h"
#include "Utility.h"
#include "CListCtrlEx.h"

#include "MemoryCard.h"
#include "MemoryCardFactory.h"

#include "SaveFileBase.h"
#include "SaveFileFolder.h"
#include "SaveFilePSU.h"
#include "SaveFileNPO.h"
#include "DlgSelectFileType.h"
#include "IconSys.h"
//#include "IconRender.h"
//#include "IconTexture.h"
//#include "IconObject.h"

#include "PS1SaveData.h"

#include "InternetSession.h"
#include "GameData.h"
#include "IconRenderingManager.h"

#include "PS1SaveFileMcr.h"

using namespace std;

const string ICON_SYS_FILE_NAME = "icon.sys";
const string BROKEN_FILE_MESSAGE = "jt@C";
const string EXTENSION_PSU = "psu";
const string EXTENSION_NPO = "npo";
const string EXTENSION_PS1 = "ps psm mcd gme mcr mcr mci dat mc ddf pss";
const int MEMORY_CARD_CLUSTER_SIZE = 2048;

vector<pair<string, string> > vecCharMap;

Control::Control(CmemccppDlg& dlg) : m_Dialog(dlg)
{
	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	//m_pSoftData = boost::shared_ptr<SettingFile>(new SettingFile());
	//m_pSoftData->Load(strSoftDataFileName);
	m_pInternetSession = auto_ptr<InternetSession>(new InternetSession("memccpp"));
	if (!m_pInternetSession.get()){
		Log::Add("G[ : C^[lbgZbV쐬ł܂ł");
		return;
	}
	m_pMemoryCard = MemoryCardFactory::GetNullMemoryCard(*this);

	m_Dialog.SetText(TEXT_STATE, "ҋ@");

	vecCharMap.push_back(make_pair(":", "F"));
	
	GetDialog().GetToolTip().Create(&GetDialog(), TTS_ALWAYSTIP);
	GetDialog().GetToolTip().SetMaxTipWidth(300);
	GetDialog().GetToolTip().Activate(TRUE);
	OnChangeLanguage();

	m_pGameData = auto_ptr<GameData>(new GameData(*this));
	m_pIconRenderingManager = auto_ptr<IconRenderingManager>(new IconRenderingManager(*this));

	UpdateDataListPC();
}

Control::~Control()
{
	//m_vecGameTitleGetter.clear();
	//m_bIsExit = true;
	m_pMemoryCard = MemoryCardFactory::GetNullMemoryCard(*this);
	//IconRender::Exit();
	//m_pSoftData->Save(strSoftDataFileName);

	m_pIconRenderingManager.reset();
	m_pGameData.reset();

	Log::Stop();
}

bool Control::IsConnecting()
{
	return m_pMemoryCard->IsConnecting();
}

UINT Control::UpdateDataListPS2()
{
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	list_ctrl.DeleteAllItems();

	if (!m_pMemoryCard->IsConnecting()){
		Log::Add("G[ : [J[hڑɃ[J[hQƂ悤Ƃ܂ UpdateDataListPS2()");
		return -1;
	}

	m_Dialog.SetProgress(0.0f);
	m_Dialog.SetText(TEXT_STATE, "Z[uf[^Xg擾");

	CComboBox& combo = m_Dialog.GetComboBox(COMBO_BOX_MEMORY_CARD);
	if (m_pMemoryCard->EnumerateMemoryCardSlot() == 0){
		return -1;
	}

	m_Dialog.SetText(TEXT_USING_SIZE, "");

	m_pMemoryCard->ChangeDirectory("");

	const vector<MemoryCard::FILE_DATA> vec_directory = m_pMemoryCard->List();
	int progress = 0;
	int total_data_size = 0;
	for (vector<MemoryCard::FILE_DATA>::const_iterator it = vec_directory.begin(); it != vec_directory.end(); ++it){
		const string& dir = it->name;
		map<string, vector<string> >::iterator it_cache;
		if ((it_cache = m_mapDataCache.find(dir)) != m_mapDataCache.end()){
			list_ctrl.InsertItem(it_cache->second);
			int size = (int)Utility::ToInt(it_cache->second[LIST_COLUMN_SIZE]);
			total_data_size += size;
			continue;
		}
		bool is_ps1 = false;
		if (it->is_directory || GetDialog().GetCheckBoxBool(CHECK_BOX_FORCE_DIRECTORY)){
			//PS2̃J
			m_pMemoryCard->ChangeDirectory(dir);
			SetCurrentDirectory(Utility::GetProgramFolder().c_str());
			const vector<MemoryCard::FILE_DATA> vec_file = m_pMemoryCard->List();
			int data_size = 0;
			for (vector<MemoryCard::FILE_DATA>::const_iterator it_file = vec_file.begin(); it_file != vec_file.end(); ++it_file){
				int size = it_file->file_size;
				--size;
				size /= MEMORY_CARD_CLUSTER_SIZE;
				++size;
				size *= (MEMORY_CARD_CLUSTER_SIZE / 1024);
				data_size += size;
			}
			if (m_pMemoryCard->Get(ICON_SYS_FILE_NAME)){
				//PS2ʏt@C
				IconSys icon(ICON_SYS_FILE_NAME);
				if (!icon.IsOpen()){
					//icon.sysj
					vector<string> vec_element;
					vec_element = CombineItem(BROKEN_FILE_MESSAGE, dir, data_size, SAVE_DATA_TYPE_PS2);
					list_ctrl.InsertItem(vec_element);
					m_mapDataCache.insert(make_pair(dir, vec_element));
					total_data_size += data_size;
					continue;
				}
				vector<string> vec_element;
				vec_element = CombineItem(icon.GetTitle(), dir, data_size, SAVE_DATA_TYPE_PS2);
				m_mapDataCache.insert(make_pair(dir, vec_element));
				list_ctrl.InsertItem(vec_element);
				DeleteFile(ICON_SYS_FILE_NAME.c_str());
				total_data_size += data_size;
			} else {
				if (!vec_file.empty() && vec_file.size() == 1 && vec_file.front().file_size % 8192 == 0 && vec_file.front().name == it->name){
					//PS1f[^
					is_ps1 = true;
				} else {
					//PS2jt@C
					vector<string> vec_element;
					vec_element = CombineItem(BROKEN_FILE_MESSAGE, dir, data_size, SAVE_DATA_TYPE_PS2);
					list_ctrl.InsertItem(vec_element);
					m_mapDataCache.insert(make_pair(dir, vec_element));
					total_data_size += data_size;
				}
			}
		} else {
			is_ps1 = true;
		}
		if (is_ps1){
			//PS1f[^orJ
			total_data_size += 8;
			const string& file_name = it->name;
			m_pMemoryCard->Get(file_name);
			const string save_data_file_path = Utility::GetProgramFolder() + "\\" + file_name;
			PS1SaveData save_data(save_data_file_path);

			if (save_data.IsOpen()){
				const vector<string> vec_element = CombineItem(save_data.GetSaveDataName(), dir, 8, SAVE_DATA_TYPE_PS1);
				m_mapDataCache.insert(make_pair(dir, vec_element));
				list_ctrl.InsertItem(vec_element);
				m_pIconRenderingManager->AddIcon(save_data_file_path, file_name, false);
				//IconRender::AddIcon(boost::shared_ptr<IconObject>(new IconObject(boost::shared_ptr<IconTexture>(new IconTexture(save_data.GetTextureData(), 16, 16)))), dir);
			} else {
				vector<string> vec_element = CombineItem(BROKEN_FILE_MESSAGE, dir, 8, SAVE_DATA_TYPE_PS1);
				list_ctrl.InsertItem(vec_element);
				m_mapDataCache.insert(make_pair(dir, vec_element));
			}

			DeleteFile(file_name.c_str());
		}

		m_Dialog.SetProgress((float)(++progress) / (float)vec_directory.size());
	}
	m_Dialog.SetText(TEXT_USING_SIZE, (boost::format("%1%KBgp") % total_data_size).str());
	m_Dialog.SetProgress(1.0f);
	return 0;
}

UINT Control::UpdateDataListPC()
{
	m_Dialog.SetText(TEXT_STATE, "[JtH_擾");
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	list_ctrl.DeleteAllItems();
	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	CFileFind finder;
	const string folder = m_Dialog.GetEditString(EDIT_FOLDER);
	if (!Utility::IsFolder(folder)){
		CreateDirectory(folder.c_str(), NULL);
	}
	BOOL working = finder.FindFile((folder + "\\*").c_str());
	while (working){
		working = finder.FindNextFile();
		if (finder.IsDots()){
			continue;
		}
		if (finder.IsDirectory()){
			const string folder_path = (LPCTSTR)finder.GetFilePath();
			const string file_path = folder_path + "\\" + ICON_SYS_FILE_NAME;

			CFileFind finder2;
			BOOL working2 = finder2.FindFile((folder_path + "\\*.*").c_str());
			int data_size = 0;
			while (working2){
				working2 = finder2.FindNextFile();
				if (finder2.IsDots()){
					continue;
				}
				int size = (int)finder2.GetLength();
				--size;
				size /= MEMORY_CARD_CLUSTER_SIZE;
				++size;
				size *= (MEMORY_CARD_CLUSTER_SIZE / 1024);
				data_size += size;
			}

			IconSys icon(file_path);
			if (icon.IsOpen()){
				list_ctrl.InsertItem(CombineItem(icon.GetTitle(), (LPCTSTR)finder.GetFileName(), data_size, SAVE_DATA_TYPE_PS2));
			} else {
				list_ctrl.InsertItem(CombineItem(BROKEN_FILE_MESSAGE, (LPCTSTR)finder.GetFileName(), data_size, SAVE_DATA_TYPE_PS2));
			}
			continue;
		}

		//PS1f[^
		const string file_name = (LPCTSTR)finder.GetFileName();
		const string file_path = (LPCTSTR)finder.GetFilePath();
		PS1SaveData save_data(file_path);

		if (save_data.IsOpen()){
			const vector<string> vec_element = CombineItem(save_data.GetSaveDataName(), file_name, 8, SAVE_DATA_TYPE_PS1);
			m_mapDataCache.insert(make_pair(file_name, vec_element));
			list_ctrl.InsertItem(vec_element);
			m_pIconRenderingManager->AddIcon(file_path, file_name, false);
			//IconRender::AddIcon(boost::shared_ptr<IconObject>(new IconObject(boost::shared_ptr<IconTexture>(new IconTexture(save_data.GetTextureData(), 16, 16)))), file_name);
		} else {
			vector<string> vec_element = CombineItem(BROKEN_FILE_MESSAGE, file_name, 8, SAVE_DATA_TYPE_PS1);
			list_ctrl.InsertItem(vec_element);
			m_mapDataCache.insert(make_pair(file_name, vec_element));
		}
	}
	return 0;
}

void Control::UpdateMemoryCardList()
{
	CComboBox& combo = m_Dialog.GetComboBox(COMBO_BOX_MEMORY_CARD);
	combo.ResetContent();
	if (!m_pMemoryCard->IsConnecting()){
		Log::Add("x : [J[hڑɃ[J[hQƂ悤Ƃ܂ UpdateMemoryCardList()");
		return;
	}
	m_Dialog.SetText(TEXT_STATE, "[J[h");
	for (int slot = 0; slot < 2; ++slot){
		if (m_pMemoryCard->EnumerateMemoryCardSlot() & (1 << slot)){
			combo.AddString(Utility::ToString(slot).c_str());
		}
	}
	if (combo.GetCount() > 0){
		combo.SetCurSel(0);
	} else {
		return;
	}
	//m_pMemoryCard->SetMemoryCardSlot((int)Utility::ToInt(m_Dialog.GetComboBoxString(COMBO_BOX_MEMORY_CARD)));
}

const std::vector<std::string> Control::CombineItem(const std::string& name, const std::string& directory, int data_size, int type)
{
	string temp_name = Utility::ToHalf(name);
	size_t index_new_line = -1;
	while ((index_new_line = temp_name.find("\r\n", index_new_line + 1)) != string::npos){
		temp_name.replace(index_new_line, 2, " ");
	}
	index_new_line = -1;
	while ((index_new_line = temp_name.find("\n", index_new_line + 1)) != string::npos){
		if (index_new_line != 0 && IsSJIS_Kanji(temp_name[index_new_line - 1])){
			continue;
		}
		temp_name.replace(index_new_line, 1, " ");
	}
	if (temp_name[0] == ' '){
		temp_name.erase(0, temp_name.find_first_not_of(' '));
	}
	if (temp_name[temp_name.size() - 1] == ' '){
		size_t offset = temp_name.find_last_not_of(' ');
		if (IsSJIS_Kanji(temp_name[offset])){
			++offset;
		}
		++offset;
		temp_name.erase(offset, string::npos);
	}
	size_t index_space;
	while ((index_space = temp_name.find("  ")) != string::npos){
		temp_name.erase(index_space, 1);
	}

	string temp_directory;
	if (directory.find(' ')){
		temp_directory = directory.substr(directory.find_last_of(' ') + 1);
	} else {
		temp_directory = directory;
	}

	vector<string> vec_res;
	vec_res.resize(NUMBER_LIST_COLUMN);
	vec_res[LIST_COLUMN_NAME] = temp_name;
	vec_res[LIST_COLUMN_DIRECTORY] = temp_directory;
	vec_res[LIST_COLUMN_SIZE] = Utility::ToString(data_size, 4);
	vec_res[LIST_COLUMN_TYPE] = string("PS") + Utility::ToString(type);
	return vec_res;
}

void Control::SplitItem(const std::vector<std::string>& item, std::string& name, std::string& directory, int& data_size, int& type)
{
	name = item[LIST_COLUMN_NAME];
	directory =item[LIST_COLUMN_DIRECTORY];
	type = (int)Utility::ToInt(item[LIST_COLUMN_TYPE].substr(2));
}

UINT Control::OnConnectFtp()
{
	m_Dialog.EnableWindow(FALSE);
	m_Dialog.SetText(TEXT_STATE, "FTPڑJn");

	const string server_name = m_Dialog.GetEditString(EDIT_ADDRESS);
	if (!server_name.empty()){
		m_pMemoryCard = MemoryCardFactory::GetMemoryCard(*this, MemoryCardFactory::CONNECTION_TYPE_FTP, server_name);
	}
	UpdateMemoryCardList();
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnConnectPS2Link()
{
	m_Dialog.EnableWindow(FALSE);
	m_Dialog.SetText(TEXT_STATE, "PS2LinkڑJn(΂炭҂)");

	const string server_name = m_Dialog.GetEditString(EDIT_ADDRESS);
	if (!server_name.empty()){
		m_pMemoryCard = MemoryCardFactory::GetMemoryCard(*this, MemoryCardFactory::CONNECTION_TYPE_PS2LINK, server_name);
	}

	UpdateMemoryCardList();
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnDisconnect()
{
	m_Dialog.EnableWindow(FALSE);

	m_pMemoryCard = MemoryCardFactory::GetNullMemoryCard(*this);

	m_mapDataCache.clear();
	UpdateMemoryCardList();
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnExport()
{
	m_Dialog.EnableWindow(FALSE);
	const string folder_path = Utility::GetFolderNameFromWindow(GetDialog().GetSafeHwnd(), Utility::GetProgramFolder());
	if (folder_path.empty()){
		m_Dialog.EnableWindow(TRUE);
		return -1;
	}
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	bool found = false;
	int count = list_ctrl.GetItemCount();
	for (int i = 0; i < count; ++i){
		if (list_ctrl.GetCheck(i) != 0){
			found = true;
			break;
		}
	}
	if (!found){
		return -1;
	}
	const string local_folder = m_Dialog.GetEditString(EDIT_FOLDER);
	vector<vector<string> > vec_task;
	vec_task.reserve(count);
	int count_save_data[3];
	memset(count_save_data, 0, sizeof(count_save_data));
	for (int row = 0; row < count; ++row){
		if (list_ctrl.GetCheck(row) == 0){
			continue;
		}
		vec_task.push_back(list_ctrl.GetItem(row));
		string name, directory;
		int data_size, type;
		SplitItem(vec_task.back(), name, directory, data_size, type);
		++count_save_data[type];
	}
	if (!((count_save_data[SAVE_DATA_TYPE_PS1] == 0) ^ (count_save_data[SAVE_DATA_TYPE_PS2] == 0))){
	}
	int progress = 0;
	m_Dialog.SetProgress(0.0f);
	DlgSelectFileType dlg;
	if (dlg.DoModal() != IDOK){
		return -1;
	}
	for (vector<vector<string> >::iterator it = vec_task.begin(); it != vec_task.end(); ++it){
		if (count_save_data[SAVE_DATA_TYPE_PS1]){
			//PS1
			const string& name = (*it)[LIST_COLUMN_NAME];
			const string& directory = (*it)[LIST_COLUMN_DIRECTORY];

			m_Dialog.SetText(TEXT_STATE, "Ǎ : " + name);
			PS1SaveFileMcr mcr;
			mcr.LoadRaw(local_folder + "\\" + directory);
			m_Dialog.SetText(TEXT_STATE, " : " + name);
			mcr.Save(folder_path + "\\" + directory + ".mcr");
		} else {
			//PS2
			const string& name = (*it)[LIST_COLUMN_NAME];
			const string& directory = (*it)[LIST_COLUMN_DIRECTORY];

			m_Dialog.SetText(TEXT_STATE, "Ǎ : " + name);
			SaveFileFolder file_folder;
			file_folder.Load(local_folder + "\\" + directory);
			m_Dialog.SetText(TEXT_STATE, " : " + name);
			auto_ptr<SaveFileBase> file_to;
			if (dlg.GetFileType() == FILE_TYPE_PSU){
				file_to = auto_ptr<SaveFilePSU>(new SaveFilePSU());
			} else if (dlg.GetFileType() == FILE_TYPE_NPO){
				file_to = auto_ptr<SaveFileNPO>(new SaveFileNPO());
			} else {
				return -1;
			}
			file_to->SetData(file_folder.GetData());
			file_to->Save(folder_path);

			//list_ctrl.SetCheck(row, FALSE);
		}
		m_Dialog.SetProgress((float)(++progress) / (float)vec_task.size());
	}
	m_Dialog.SetProgress(1.0f);
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

const string FILE_TYPE_FILTER = "PSUt@C (*.psu)|*.psu|npot@C (*.npo)|*.npo|PS1Z[uf[^|*.ps; *.psm; *.mcd; *.gme; *.mcr; *.mcr; *.mci; *.dat; *.mc; *.ddf; *.pss||";

UINT Control::OnImport()
{
	Log::Add("Control::OnImport()");
	m_Dialog.EnableWindow(FALSE);
	const vector<string> vec_file_path = Utility::GetFileNameFromWindow(true, true, "", "", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, FILE_TYPE_FILTER, &m_Dialog);
	Log::Add("It@C : " + Utility::ToString(vec_file_path.size()));
	const string local_folder = m_Dialog.GetEditString(EDIT_FOLDER);
	int progress = 0;
	m_Dialog.SetProgress(0.0f);
	for (vector<string>::const_iterator it = vec_file_path.begin(); it != vec_file_path.end(); ++it){
		m_Dialog.SetText(TEXT_STATE, "Ǎ : " + *it);
		const string file_extension = Utility::ToLower(Utility::SplitFileExtention(*it));
		auto_ptr<SaveFileBase> file_from;
		if (file_extension == EXTENSION_PSU){
			Log::Add("Ǎ(psu) : " + *it);
			file_from = auto_ptr<SaveFilePSU>(new SaveFilePSU);
		} else if (file_extension == EXTENSION_NPO){
			Log::Add("Ǎ(npo) : " + *it);
			file_from = auto_ptr<SaveFileNPO>(new SaveFileNPO);
		} else if (EXTENSION_PS1.find(file_extension) != string::npos){
			Log::Add("Ǎ(ps1) : " + *it);
			PS1SaveFileMcr mcr;
			mcr.Load(*it);
			mcr.SaveRaw(local_folder);
			continue;
		} else {
			Log::Add("gq̔ʃ~X : " + file_extension + " " + *it);
			continue;
		}
		file_from->Load(*it);
		m_Dialog.SetText(TEXT_STATE, " : " + *it);

		SaveFileFolder file_folder;
		file_folder.SetData(file_from->GetData());
		file_folder.Save(local_folder);
		m_Dialog.SetProgress((float)(++progress) / (float)vec_file_path.size());
	}
	m_Dialog.SetProgress(1.0f);
	UpdateDataListPC();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnMovePS2ToPC()
{
	OnCopyPS2ToPC();
	OnDeletePS2();
	return 0;
}

UINT Control::OnMovePCToPS2()
{
	OnCopyPCToPS2();
	OnDeletePC();
	return 0;
}

UINT Control::OnCopyPS2ToPC()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	int count = list_ctrl.GetItemCount();
	vector<vector<string> > vec_task;
	vec_task.reserve(count);
	for (int row = 0; row < count; ++row){
		if (list_ctrl.GetCheck(row) != 0){
			vec_task.push_back(list_ctrl.GetItem(row));
		}
	}
	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	int progress = 0;
	m_Dialog.SetProgress(0.0f);
	for (vector<vector<string> >::iterator it = vec_task.begin(); it != vec_task.end(); ++it){
		string name, directory;
		int data_size, type;
		SplitItem(*it, name, directory, data_size, type);
		if (type == SAVE_DATA_TYPE_PS1){
			//PS1
			const string local_file_path = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory;
			if (Utility::IsFile(local_file_path)){
				MoveFile(local_file_path.c_str(), Utility::GenerateBackupFileName(local_file_path).c_str());
			}
			SetCurrentDirectory(m_Dialog.GetEditString(EDIT_FOLDER).c_str());
			m_pMemoryCard->ChangeDirectory(directory);
			m_pMemoryCard->Get(directory);
		} else {
			//PS2
			const string local_folder = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory;
			if (Utility::IsFolder(local_folder)){
				MoveFile(local_folder.c_str(), Utility::GenerateBackupFileName(local_folder).c_str());
			}
			CreateDirectory(local_folder.c_str(), NULL);
			SetCurrentDirectory(local_folder.c_str());
			m_pMemoryCard->ChangeDirectory(directory);
			const vector<MemoryCard::FILE_DATA> vec_file = m_pMemoryCard->List();
			for (vector<MemoryCard::FILE_DATA>::const_iterator it_file = vec_file.begin(); it_file != vec_file.end(); ++it_file){
				m_Dialog.SetText(TEXT_STATE, "_E[h : " + name + "\n" + directory + "/" + it_file->name.c_str());
				m_pMemoryCard->Get(it_file->name.c_str());
				CFileStatus file_status;
				const string local_file_path = local_folder + "\\" + it_file->name;
				CFile::GetStatus(local_file_path.c_str(), file_status);
				file_status.m_ctime = it_file->time;
				//t̕b̕Ǘԍɂ
				file_status.m_ctime -= CTimeSpan(0, 0, 0, file_status.m_ctime.GetSecond());
				file_status.m_ctime += CTimeSpan(0, 0, 0, (int)distance(vec_file.begin(), it_file) + 1);
				file_status.m_mtime = it_file->time;
				file_status.m_atime = it_file->time;
				CFile::SetStatus(local_file_path.c_str(), file_status);
			}
		}
		m_Dialog.SetProgress((float)(++progress) / (float)vec_task.size());
	}
	m_Dialog.SetProgress(1.0f);
	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	UpdateDataListPC();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

boost::regex regexBackupFolderNameMatch(".*\\.[0-9]{3}");

UINT Control::OnCopyPCToPS2()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	int count = list_ctrl.GetItemCount();
	vector<vector<string> > vec_task;
	vec_task.reserve(count);
	for (int row = 0; row < count; ++row){
		if (list_ctrl.GetCheck(row) == 0){
			continue;
		}
		vec_task.push_back(list_ctrl.GetItem(row));
	}
	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	int progress = 0;
	m_Dialog.SetProgress(0.0f);
	for (vector<vector<string> >::iterator it = vec_task.begin(); it != vec_task.end(); ++it){
		//u.vtĂꍇ͕ʏ
		string name, directory;
		int data_size, type;
		SplitItem(*it, name, directory, data_size, type);
		if (type == SAVE_DATA_TYPE_PS1){
			string file_name = directory;
			if (file_name.size() > 4){
				if (boost::regex_match(file_name, regexBackupFolderNameMatch)){
					file_name.erase(file_name.size() - 4);
				}
			}
			const string file_path_from = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory;
			const string file_path_to = Utility::GetProgramFolder() + "\\" + file_name;
			CopyFile(file_path_from.c_str(), file_path_to.c_str(), FALSE);
			SetCurrentDirectory(m_Dialog.GetEditString(EDIT_FOLDER).c_str());
			m_pMemoryCard->ChangeDirectory("");
			m_pMemoryCard->MakeDirectory(file_name);
			m_pMemoryCard->ChangeDirectory(file_name);
			m_Dialog.SetText(TEXT_STATE, "Abv[h : " + name + "\n" + directory);
			m_pMemoryCard->Put(file_name);
			map<string, vector<string> >::iterator it_cache;
			if ((it_cache = m_mapDataCache.find(file_name)) != m_mapDataCache.end()){
				m_mapDataCache.erase(it_cache);
			}
		} else {
			if (directory.size() > 4){
				if (boost::regex_match(directory, regexBackupFolderNameMatch)){
					directory.erase(directory.size() - 4);
				}
			}
			const string local_folder = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory;
			SetCurrentDirectory(local_folder.c_str());
			m_pMemoryCard->ChangeDirectory("");
			m_pMemoryCard->MakeDirectory(directory);
			m_pMemoryCard->ChangeDirectory(directory);
			CFileFind finder;
			BOOL working = finder.FindFile((local_folder + "\\*.*").c_str());
			while (working){
				working = finder.FindNextFile();
				if (finder.IsDots() || finder.IsDirectory()){
					continue;
				}
				m_Dialog.SetText(TEXT_STATE, "Abv[h : " + name + "\n" + directory + "/" + (LPCTSTR)finder.GetFileName());
				m_pMemoryCard->Put((LPCTSTR)finder.GetFileName());
			}
			map<string, vector<string> >::iterator it_cache;
			if ((it_cache = m_mapDataCache.find(directory)) != m_mapDataCache.end()){
				m_mapDataCache.erase(it_cache);
			}
		}
		m_Dialog.SetProgress((float)(++progress) / (float)vec_task.size());
	}
	m_Dialog.SetProgress(1.0f);
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnDeletePS2()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	m_pMemoryCard->ChangeDirectory("");
	int count = list_ctrl.GetItemCount();
	vector<int> vec_index;
	vec_index.reserve(count);
	for (int row = 0; row < count; ++row){
		if (list_ctrl.GetCheck(row) == 0){
			continue;
		}
		vec_index.push_back(row);
	}
	int progress = 0;
	m_Dialog.SetProgress(0.0f);
	for (vector<int>::iterator it = vec_index.begin(); it != vec_index.end(); ++it){
		string name, directory;
		int data_size, type;
		SplitItem(list_ctrl.GetItem(*it), name, directory, data_size, type);
		if (m_pMemoryCard->ChangeDirectory(directory)){
			const vector<MemoryCard::FILE_DATA> vec_file = m_pMemoryCard->List();
			for (vector<MemoryCard::FILE_DATA>::const_iterator it = vec_file.begin(); it != vec_file.end(); ++it){
				m_Dialog.SetText(TEXT_STATE, "폜 : " + name + "\n" + directory + "/" + it->name.c_str());
				m_pMemoryCard->Remove(it->name.c_str());
			}
			m_pMemoryCard->ChangeDirectory("");
			m_pMemoryCard->RemoveDirectory(directory);
		} else {
			m_Dialog.SetText(TEXT_STATE, "폜 : " + name + "\n" + directory);
			m_pMemoryCard->Remove(directory);
		}
		map<string, vector<string> >::iterator it_cache;
		if ((it_cache = m_mapDataCache.find(directory)) != m_mapDataCache.end()){
			m_mapDataCache.erase(it_cache);
		}
		m_Dialog.SetProgress((float)(++progress) / (float)vec_index.size());
	}
	m_Dialog.SetProgress(1.0f);
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnDeletePC()
{
	//m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	int count = list_ctrl.GetItemCount();
	for (int row = 0; row < count; ++row){
		if (list_ctrl.GetCheck(row) == 0){
			continue;
		}
		string name, directory;
		int data_size, type;
		SplitItem(list_ctrl.GetItem(row), name, directory, data_size, type);
		const string folder = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory;
		m_Dialog.SetText(TEXT_STATE, "폜 : " + name + "\n" + folder);
		if (Utility::IsFolder(folder)){
			Utility::DeleteFolder(folder);
		} else {
			DeleteFile(folder.c_str());
		}
	}
	UpdateDataListPC();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	//m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnChangePS2()
{
	m_Dialog.EnableWindow(FALSE);
	UpdateGameData(LIST_CTRL_PS2);
	UpdateSaveData(LIST_CTRL_PS2);

	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	int index = list_ctrl.GetNextItem(-1, LVIS_SELECTED);
	if (index == -1){
		m_Dialog.EnableWindow(TRUE);
		return 0;
	}
	string name, directory;
	int data_size, type;
	SplitItem(list_ctrl.GetItem(index), name, directory, data_size, type);
	//if (IconRender::IsCached(directory)){
	//	IconRender::SetIcon("", directory);
	//	IconRender::Draw();
	//}
	if (m_pIconRenderingManager->IsCached(directory)){
		m_pIconRenderingManager->SetIcon("", directory, true);
	} else if (type == SAVE_DATA_TYPE_PS2){
		CListCtrlEx& list_ctrl_pc = m_Dialog.GetListCtrl(LIST_CTRL_PC);
		bool found_in_pc = false;
		string pc_name, pc_directory;
		for (int row = 0; row < list_ctrl_pc.GetItemCount(); ++row){
			int data_size, type;
			SplitItem(list_ctrl_pc.GetItem(row), pc_name, pc_directory, data_size, type);
			if (pc_directory.find(directory) != string::npos){
				found_in_pc = true;
				break;
			}
		}
		if (found_in_pc){
			const string icon_sys_path = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + pc_directory + "\\icon.sys";
			m_pIconRenderingManager->SetIcon(icon_sys_path, pc_directory, true);
			//IconRender::SetIcon(icon_sys_path, pc_directory);
			//directory != pc_directorỷ\l
		} else {
			SetCurrentDirectory(Utility::GetProgramFolder().c_str());
			m_pMemoryCard->ChangeDirectory(directory);
			if (m_pMemoryCard->Get("icon.sys")){
				IconSys icon_sys("icon.sys");
				const string icon_file_name = icon_sys.GetIconFileName(0);
				m_pMemoryCard->Get(icon_file_name);
				m_pIconRenderingManager->SetIcon(Utility::GetProgramFolder() + "icon.sys", directory, true);
				//IconRender::SetIcon(Utility::GetProgramFolder() + "icon.sys", directory);
				DeleteFile(icon_file_name.c_str());
				DeleteFile("icon.sys");
			}
		}
		//IconRender::Draw();
	} else {
		Log::Add("Control::OnChangePS2() : LbVĂȂPS1݂܂");
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnChangePC()
{
//	m_Dialog.EnableWindow(FALSE);
	UpdateGameData(LIST_CTRL_PC);
	UpdateSaveData(LIST_CTRL_PC);

	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	int index = list_ctrl.GetNextItem(-1, LVIS_SELECTED);
	if (index == -1){
		m_Dialog.EnableWindow(TRUE);
		return 0;
	}
	string name, directory;
	int data_size, type;
	SplitItem(list_ctrl.GetItem(index), name, directory, data_size, type);
	//if (IconRender::IsCached(directory)){
	if (m_pIconRenderingManager->IsCached(directory)){
		m_pIconRenderingManager->SetIcon("", directory, true);
		//IconRender::SetIcon("", directory);
		//IconRender::Draw();
	} else if (type == SAVE_DATA_TYPE_PS2){
		const string icon_sys_path = m_Dialog.GetEditString(EDIT_FOLDER) + "\\" + directory + "\\icon.sys";
		m_pIconRenderingManager->SetIcon(icon_sys_path, directory, true);
		//IconRender::SetIcon(icon_sys_path, directory);
		//IconRender::Draw();
	}

//	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnChangeMemoryCard()
{
	m_Dialog.EnableWindow(FALSE);
	//m_pMemoryCard->SetMemoryCardSlot((int)Utility::ToInt(m_Dialog.GetComboBoxString(COMBO_BOX_MEMORY_CARD)));
	UpdateDataListPS2();
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

void Control::UpdateGameData(int index)
{
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(index);
	int i = list_ctrl.GetNextItem(-1, LVIS_SELECTED);
	if (i == -1){
		return;
	}
	string name, directory;
	int data_size, type;
	SplitItem(list_ctrl.GetItem(i), name, directory, data_size, type);
	m_pGameData->Show(directory);
	//boost::shared_ptr<GameTitleGetter> title_getter(new GameTitleGetter(directory, ++m_iGameDataDispIndexDistribute));
	//if (!title_getter->Run()){
	//	m_vecGameTitleGetter.push_back(title_getter);
	//}
}

void Control::UpdateSaveData(int index)
{
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(index);
	int i = list_ctrl.GetNextItem(-1, LVIS_SELECTED);
	if (i == -1){
		return;
	}
	string name, directory;
	int data_size, type;
	SplitItem(list_ctrl.GetItem(i), name, directory, data_size, type);
	//t擾
	CTime ctime_date = CTime::GetCurrentTime();
	if (index == LIST_CTRL_PC){
		CFileStatus file_status;
		if (CFile::GetStatus((GetDialog().GetEditString(EDIT_FOLDER) + "\\" + directory + "\\icon.sys").c_str(), file_status)){
			ctime_date = file_status.m_ctime;
		}
	} else if (index == LIST_CTRL_PS2){
		m_pMemoryCard->ChangeDirectory(directory);
		const vector<MemoryCard::FILE_DATA> vec_file = m_pMemoryCard->List();
		vector<MemoryCard::FILE_DATA>::const_iterator it;
		for (it = vec_file.begin(); it != vec_file.end(); ++it){
			if (it->name == "icon.sys"){
				ctime_date = it->time;
				break;
			}
		}
	}
	string date = (LPCTSTR)ctime_date.Format("%Y/%m/%d %H:%M:%S");
	m_Dialog.SetText(TEXT_SAVE_DATA, name + "\r\n" + directory + "\r\n" + date);
}

UINT Control::OnSelectAllPS2()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	int count = list_ctrl.GetItemCount();
	for (int row = 0; row < count; ++row){
		list_ctrl.SetCheck(row, 1);
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnSelectAllPC()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	int count = list_ctrl.GetItemCount();
	for (int row = 0; row < count; ++row){
		list_ctrl.SetCheck(row, 1);
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnSelectClearPS2()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PS2);
	int count = list_ctrl.GetItemCount();
	for (int row = 0; row < count; ++row){
		list_ctrl.SetCheck(row, 0);
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnSelectClearPC()
{
	m_Dialog.EnableWindow(FALSE);
	CListCtrlEx& list_ctrl = m_Dialog.GetListCtrl(LIST_CTRL_PC);
	int count = list_ctrl.GetItemCount();
	for (int row = 0; row < count; ++row){
		list_ctrl.SetCheck(row, 0);
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnFolderRef()
{
	m_Dialog.EnableWindow(FALSE);
	const string folder_path = Utility::GetFolderNameFromWindow(GetDialog().GetSafeHwnd(), GetDialog().GetEditString(EDIT_FOLDER));
	if (!folder_path.empty()){
		m_Dialog.GetEdit(EDIT_FOLDER).SetWindowText(folder_path.c_str());
		UpdateDataListPC();
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnUSBRef()
{
	m_Dialog.EnableWindow(FALSE);
	string folder_path = Utility::GetFolderNameFromWindow(GetDialog().GetSafeHwnd(), GetDialog().GetEditString(EDIT_USB_FOLDER));
	if (::GetDriveType(folder_path.substr(0, 3).c_str()) == DRIVE_REMOVABLE){
		GetDialog().GetEdit(EDIT_USB_FOLDER).SetWindowText(folder_path.c_str());
	}
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnConnectUSBMemory()
{
	m_Dialog.EnableWindow(FALSE);
	m_Dialog.SetText(TEXT_STATE, "USB[ڑJn");

	m_pMemoryCard = MemoryCardFactory::GetMemoryCard(*this, MemoryCardFactory::CONNECTION_TYPE_USB_MEMORY);

	UpdateMemoryCardList();
	UpdateDataListPS2();
	m_Dialog.SetText(TEXT_STATE, "ҋ@");
	m_Dialog.EnableWindow(TRUE);
	return 0;
}

UINT Control::OnDropFilePath()
{
	GetDialog().EnableWindow(FALSE);
	const std::string local_folder = GetDialog().GetEditString(EDIT_FOLDER);
	for (vector<string>::iterator it = m_vecDropFilePath.begin(); it != m_vecDropFilePath.end(); ++it){
		if (!Utility::IsFile(*it)){
			continue;
		}
		string extention = Utility::ToLower(Utility::SplitFileExtention(*it));
		auto_ptr<SaveFileBase> file_from;
		if (extention == EXTENSION_PSU){
			file_from = auto_ptr<SaveFilePSU>(new SaveFilePSU);
		} else if (extention == EXTENSION_NPO){
			file_from = auto_ptr<SaveFileNPO>(new SaveFileNPO);
		} else {
			continue;
		}
		file_from->Load(*it);
		SaveFileFolder folder;
		folder.SetData(file_from->GetData());
		folder.Save(local_folder);
	}
	UpdateDataListPC();
	m_vecDropFilePath.clear();
	GetDialog().EnableWindow(TRUE);
	return 0;
}

const string strSectionDialog = "Dialog";
const string strSectionToolTip = "ToolTip";

const string strKeyStatic = "STATIC";
const string strKeyGroup = "GROUP";
const string strKeyButton = "BUTTON";
const string strKeyCheckBox = "CHECKBOX";
const string strKeyEdit = "EDIT";
const string strKeyComboBox = "COMBOBOX";

UINT Control::OnChangeLanguage()
{
	const string file_name = "language\\" + GetDialog().GetComboBoxString(COMBO_BOX_LANGUAGE) + ".ini";
	SettingFile language_file;
	language_file.Load(file_name);
	for (int i = 0; i < NUMBER_STATIC; ++i){
		GetDialog().GetStatic(i).SetWindowText(language_file.Get(strSectionDialog, strKeyStatic + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_GROUP; ++i){
		GetDialog().GetGroup(i).SetWindowText(language_file.Get(strSectionDialog, strKeyGroup + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_BUTTON; ++i){
		GetDialog().GetButton(i).SetWindowText(language_file.Get(strSectionDialog, strKeyButton + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_CHECK_BOX; ++i){
		GetDialog().GetCheckBox(i).SetWindowText(language_file.Get(strSectionDialog, strKeyCheckBox + Utility::ToString(i)).c_str());
	}

	CToolTipCtrl& tool_tip = GetDialog().GetToolTip();
	for (int i = 0; i < NUMBER_BUTTON; ++i){
		tool_tip.AddTool(&GetDialog().GetButton(i), language_file.Get(strSectionToolTip, strKeyButton + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_CHECK_BOX; ++i){
		tool_tip.AddTool(&GetDialog().GetCheckBox(i), language_file.Get(strSectionToolTip, strKeyCheckBox + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_EDIT; ++i){
		tool_tip.AddTool(&GetDialog().GetEdit(i), language_file.Get(strSectionToolTip, strKeyEdit + Utility::ToString(i)).c_str());
	}
	for (int i = 0; i < NUMBER_COMBO_BOX; ++i){
		tool_tip.AddTool(&GetDialog().GetComboBox(i), language_file.Get(strSectionToolTip, strKeyComboBox + Utility::ToString(i)).c_str());
	}
	return 0;
}

void Control::SetDropFilePath(const std::vector<std::string>& vec_file_path)
{
	m_vecDropFilePath = vec_file_path;
}

CInternetSession* Control::GetInternetSession()
{
	return m_pInternetSession->GetSession();
}

//XbhvV[W
UINT Control::UpdateDataListPS2ThreadProc(LPVOID param)	{return ((Control*)param)->UpdateDataListPS2();}

UINT Control::UpdateDataListPCThreadProc(LPVOID param)	{return ((Control*)param)->UpdateDataListPC();}

UINT Control::OnConnectFtpThreadProc(LPVOID param)		{return ((Control*)param)->OnConnectFtp();}

UINT Control::OnConnectPS2LinkThreadProc(LPVOID param)	{return ((Control*)param)->OnConnectPS2Link();}

UINT Control::OnDisconnectThreadProc(LPVOID param)		{return ((Control*)param)->OnDisconnect();}

UINT Control::OnExportThreadProc(LPVOID param)			{return ((Control*)param)->OnExport();}

UINT Control::OnImportThreadProc(LPVOID param)			{return ((Control*)param)->OnImport();}

UINT Control::OnMovePS2ToPCThreadProc(LPVOID param)		{return ((Control*)param)->OnMovePS2ToPC();}

UINT Control::OnMovePCToPS2ThreadProc(LPVOID param)		{return ((Control*)param)->OnMovePCToPS2();}

UINT Control::OnCopyPS2ToPCThreadProc(LPVOID param)		{return ((Control*)param)->OnCopyPS2ToPC();}

UINT Control::OnCopyPCToPS2ThreadProc(LPVOID param)		{return ((Control*)param)->OnCopyPCToPS2();}

UINT Control::OnDeletePS2ThreadProc(LPVOID param)		{return ((Control*)param)->OnDeletePS2();}

UINT Control::OnDeletePCThreadProc(LPVOID param)		{return ((Control*)param)->OnDeletePC();}

UINT Control::OnChangePS2ThreadProc(LPVOID param)		{return ((Control*)param)->OnChangePS2();}

UINT Control::OnChangePCThreadProc(LPVOID param)		{return ((Control*)param)->OnChangePC();}

UINT Control::OnChangeMemoryCardThreadProc(LPVOID param){return ((Control*)param)->OnChangeMemoryCard();}

UINT Control::OnSelectAllPS2ThreadProc(LPVOID param)	{return ((Control*)param)->OnSelectAllPS2();}

UINT Control::OnSelectAllPCThreadProc(LPVOID param)		{return ((Control*)param)->OnSelectAllPC();}

UINT Control::OnSelectClearPS2ThreadProc(LPVOID param)	{return ((Control*)param)->OnSelectClearPS2();}

UINT Control::OnSelectClearPCThreadProc(LPVOID param)	{return ((Control*)param)->OnSelectClearPC();}

UINT Control::OnFolderRefThreadProc(LPVOID param)		{return ((Control*)param)->OnFolderRef();}

UINT Control::OnUSBRefThreadProc(LPVOID param)			{return ((Control*)param)->OnUSBRef();}

UINT Control::OnConnectUSBMemoryThreadProc(LPVOID param){return ((Control*)param)->OnConnectUSBMemory();}

UINT Control::OnDropFilePathThreadProc(LPVOID param)	{return ((Control*)param)->OnDropFilePath();}

UINT Control::OnChangeLanguageThreadProc(LPVOID param)	{return ((Control*)param)->OnChangeLanguage();}

// PoCgڂۂ𔻒f
bool Control::IsSJIS_Kanji(char ch)
{
    // 0x81 - 0x9F  0xE0 - 0xFC ͈̔
    return _ismbblead( (WORD)ch) != 0 ? TRUE:FALSE;
}
