#ifndef CONTROL_H
#define CONTROL_H

#include <utility>
#include <vector>
#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include "MemoryCard.h"

class CmemccppDlg;
class MemoryCard;
class InternetSession;
class GameData;
class CInternetSession;
class IconRenderingManager;

const std::string strSoftDataFileName = "softdata.ini";

enum SAVE_DATA_TYPE{
	SAVE_DATA_TYPE_PS1 = 1,
	SAVE_DATA_TYPE_PS2 = 2
};

class Control{
private:
	CmemccppDlg& m_Dialog;
	boost::shared_ptr<MemoryCard> m_pMemoryCard;
	std::vector<std::string> m_vecDropFilePath;

	std::auto_ptr<InternetSession> m_pInternetSession;

	void UpdateMemoryCardList();
	void UpdateGameData(int index);
	void UpdateSaveData(int index);

	// PoCgڂۂ𔻒f
	bool IsSJIS_Kanji(char ch);

	const std::vector<std::string> CombineItem(const std::string& name, const std::string& directory, int data_size, int type);
	void SplitItem(const std::vector<std::string>& item, std::string& name, std::string& directory, int& data_size, int& type);
	std::map<std::string, std::vector<std::string> > m_mapDataCache;

	std::auto_ptr<GameData> m_pGameData;
	std::auto_ptr<IconRenderingManager> m_pIconRenderingManager;

public:
	Control(CmemccppDlg& dlg);
	virtual ~Control();


	static UINT UpdateDataListPS2ThreadProc(LPVOID param);
	static UINT UpdateDataListPCThreadProc(LPVOID param);
	static UINT OnConnectFtpThreadProc(LPVOID param);
	static UINT OnConnectPS2LinkThreadProc(LPVOID param);
	static UINT OnDisconnectThreadProc(LPVOID param);
	static UINT OnExportThreadProc(LPVOID param);
	static UINT OnImportThreadProc(LPVOID param);
	static UINT OnMovePS2ToPCThreadProc(LPVOID param);
	static UINT OnMovePCToPS2ThreadProc(LPVOID param);
	static UINT OnCopyPS2ToPCThreadProc(LPVOID param);
	static UINT OnCopyPCToPS2ThreadProc(LPVOID param);
	static UINT OnDeletePS2ThreadProc(LPVOID param);
	static UINT OnDeletePCThreadProc(LPVOID param);
	static UINT OnChangePS2ThreadProc(LPVOID param);
	static UINT OnChangePCThreadProc(LPVOID param);
	static UINT OnChangeMemoryCardThreadProc(LPVOID param);
	static UINT OnSelectAllPS2ThreadProc(LPVOID param);
	static UINT OnSelectAllPCThreadProc(LPVOID param);
	static UINT OnSelectClearPS2ThreadProc(LPVOID param);
	static UINT OnSelectClearPCThreadProc(LPVOID param);
	static UINT OnFolderRefThreadProc(LPVOID param);
	static UINT OnUSBRefThreadProc(LPVOID param);
	static UINT OnConnectUSBMemoryThreadProc(LPVOID param);
	static UINT OnDropFilePathThreadProc(LPVOID param);
	static UINT OnChangeLanguageThreadProc(LPVOID param);

	UINT UpdateDataListPS2();
	UINT UpdateDataListPC();
	UINT OnConnectFtp();
	UINT OnConnectPS2Link();
	UINT OnDisconnect();
	UINT OnExport();
	UINT OnImport();
	UINT OnMovePS2ToPC();
	UINT OnMovePCToPS2();
	UINT OnCopyPS2ToPC();
	UINT OnCopyPCToPS2();
	UINT OnDeletePS2();
	UINT OnDeletePC();
	UINT OnChangePS2();
	UINT OnChangePC();
	UINT OnChangeMemoryCard();
	UINT OnSelectAllPS2();
	UINT OnSelectAllPC();
	UINT OnSelectClearPS2();
	UINT OnSelectClearPC();
	UINT OnFolderRef();
	UINT OnUSBRef();
	UINT OnConnectUSBMemory();
	UINT OnDropFilePath();
	UINT OnChangeLanguage();

	bool IsConnecting();
	CmemccppDlg& GetDialog(){return m_Dialog;}
	CInternetSession* GetInternetSession();
	void SetDropFilePath(const std::vector<std::string>& vec_file_path);
};

#endif
