#include "stdafx.h"
#include <sstream>
#include <boost/regex.hpp>
#include "GameDataWorker.h"
#include "Control.h"
#include "SettingFile.h"
#include "Resource.h"
#include "memccppDlg.h"
#include "Log.h"
#include "Utility.h"
#include "InternetFile.h"
#include "GameData.h"

using namespace std;

boost::regex REGEX_DIRECTORY_NAME("B[IAE][A-Z]{4}[\\-P][0-9]{5}.*");

GameDataWorker::GameDataWorker(Control& control, const std::string& directory, GameData& game_data) : m_GameData(game_data), m_Control(control)
{
	if (!boost::regex_match(directory, REGEX_DIRECTORY_NAME)){
		return;
	}

	m_strProductID = Utility::ToLower(directory.substr(2, 4)) + directory.substr(7, 5);
}

GameDataWorker::~GameDataWorker()
{
}

void GameDataWorker::Run()
{
	if (m_strProductID.empty()){
		return;
	}

	const SettingFile& data_base = m_GameData.GetDataBase();
	if (data_base.IsInSection(m_strProductID)){
		FromLocal();
	} else {
		FromInternet();
	}
}

const string strKeyTitle = "TITLE";
const string strKeyFormat = "FORMAT";
const string strKeyGenre = "GENRE";
const string strKeyMaker = "MAKER";
const string strKeyPrice = "PRICE";
const string strKeyPriceTaxIn = "PRICE_TAX_IN";
const string strKeySaleDate = "SALE_DATE";

void GameDataWorker::FromLocal()const
{
	const SettingFile& soft = m_GameData.GetDataBase();

	string text;
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyTitle));
	text += "\r\n";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyGenre));
	text += " ";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeySaleDate));
	text += "\r\n";
	text += Utility::ToHalf(soft.Get(m_strProductID, strKeyMaker));

	ShowGameData(text);
}

const char *strURLPrefix[] = {
	"http://www.jp.playstation.com/software/title/",
	"http://www.jp.playstation.com.nyud.net:8090/software/title/"
};

const int iHttpBufferSize = 64 * 1024;
void GameDataWorker::FromInternet()const
{
	if (!m_Control.GetDialog().GetCheckBoxBool(CHECK_BOX_USE_INTERNET)){
		return;
	}

	const int index_url = m_Control.GetDialog().GetCheckBoxBool(CHECK_BOX_USE_PROXY) ? 1 : 0;
	const string url = string(strURLPrefix[index_url]) + m_strProductID + ".html";
	InternetFile file(m_Control.GetInternetSession(), url);
	if (!file.IsOpened()){
		Log::Add("GameDataWorker::FromInternet() : html擾ł܂ł");
		return;
	}

	SettingFile& soft = m_GameData.GetDataBase();

	string line;
	if (!file.ReadString(line)){
		string value = "NO DATA";
		soft.Add(m_strProductID, strKeyTitle, value);
		soft.Add(m_strProductID, strKeyMaker, value);
		soft.Add(m_strProductID, strKeyFormat, value);
		soft.Add(m_strProductID, strKeyGenre, value);
		soft.Add(m_strProductID, strKeyPrice, value);
		soft.Add(m_strProductID, strKeyPriceTaxIn, value);
		soft.Add(m_strProductID, strKeySaleDate, value);
		return;
	}

	//^Cg
	while (file.ReadString(line) && line.find("titlename") == string::npos);
	{
		istringstream iss(line);
		string title;
		getline(iss, title, '>');
		getline(iss, title, '>');
		getline(iss, title);
		size_t offset = title.find("</");
		if (offset != string::npos){
			title.erase(offset, string::npos);
		} else {
			getline(istringstream(title), title, '<');
		}
		soft.Add(m_strProductID, strKeyTitle, title);
	}

	//[J[
	while (file.ReadString(line) && line.find("licensee-name") == string::npos);
	{
		istringstream iss(line);
		string maker;
		getline(iss, maker, '>');
		getline(iss, maker, '>');
		getline(iss, maker);
		size_t offset = maker.find("</");
		if (offset != string::npos){
			maker.erase(offset, string::npos);
		} else {
			getline(istringstream(maker), maker, '<');
		}
		soft.Add(m_strProductID, strKeyMaker, maker);
	}

	SkipHtml("tH[}bg", file, 2);
	file.ReadString(line);
	soft.Add(m_strProductID, strKeyFormat, EraseTag(line));

	SkipHtml("W", file, 2);
	file.ReadString(line);
	soft.Add(m_strProductID, strKeyGenre, EraseTag(line));

	SkipHtml("]i", file, 2);
	file.ReadString(line);
	EraseTag(line);
	int price = 0;
	for (int i = 0; i < 5; ++i){
		if (isdigit(line[i])){
			price *= 10;
			price += line[i] - '0';
		}
	}
	soft.Add(m_strProductID, strKeyPrice, boost::lexical_cast<string>(price));
	soft.Add(m_strProductID, strKeyPriceTaxIn, boost::lexical_cast<string>(price * 105 / 100));


	SkipHtml("", file, 2);
	file.ReadString(line);
	soft.Add(m_strProductID, strKeySaleDate, EraseTag(line));

	//SkipHtml("^Cg", file);

	//string value;

	////^Cg
	//GetNextString(file, value);

	//if (value == "tH[}bg"){
	//	//NO DATA
	//	value = "NO DATA";
	//	soft.Add(m_strProductID, strKeyTitle, value);
	//	//value = "";
	//	soft.Add(m_strProductID, strKeyMaker, value);
	//	soft.Add(m_strProductID, strKeyFormat, value);
	//	soft.Add(m_strProductID, strKeyGenre, value);
	//	soft.Add(m_strProductID, strKeyPrice, value);
	//	soft.Add(m_strProductID, strKeyPriceTaxIn, value);
	//	soft.Add(m_strProductID, strKeySaleDate, value);
	//} else {
	//	soft.Add(m_strProductID, strKeyTitle, value);

	//	//[J[
	//	GetNextString(file, value);
	//	soft.Add(m_strProductID, strKeyMaker, value);

	//	//tH[}bg
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	soft.Add(m_strProductID, strKeyFormat, value);

	//	//W
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	soft.Add(m_strProductID, strKeyGenre, value);

	//	//]i
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	if (value.find('&') != string::npos){
	//		value.erase(value.find('&'), string::npos);
	//	}
	//	if (value.find(',') != string::npos){
	//		value.erase(value.find(','), 1);
	//	}
	//	soft.Add(m_strProductID, strKeyPrice, value);
	//	soft.Add(m_strProductID, strKeyPriceTaxIn, Utility::ToString((Utility::ToInt(value) * 105 / 100)));

	//	//
	//	GetNextString(file, value);
	//	GetNextString(file, value);
	//	soft.Add(m_strProductID, strKeySaleDate, value);
	//}

	SetCurrentDirectory(Utility::GetProgramFolder().c_str());
	soft.Save(strSoftDataFileName);
	FromLocal();
}

void GameDataWorker::ShowGameData(const std::string& text)const
{
	m_Control.GetDialog().SetText(TEXT_GAME_DATA, text);
}

bool GameDataWorker::SkipHtml(const std::string& target, InternetFile& file, int additional_line)const
{
	string s;
	while (file.ReadString(s)){
		if (s.find(target) != string::npos){
			for (int i = 0; i < additional_line; ++i){
				file.ReadString(s);
			}
			return true;
		}
	}
	return false;
}

const string strTagTD = "</TD>";

bool GameDataWorker::GetNextString(InternetFile& file, std::string& res)const
{
	string s;
	while (file.ReadString(s)){
		wstring line = Utility::StringToWString(s);
		while (!line.empty() && line[0] == L'<'){
			line.erase(0, line.find(L'>') + 1);
		}
		if (line.empty() || line.find_first_not_of(L' ') == string::npos){
			continue;
		}
		wstring temp;
		getline(wistringstream(line), temp, L'<');
		res = Utility::WStringToString(temp);
		return true;
	}
	return false;
}

const std::string& GameDataWorker::EraseTag(std::string& s)const
{
	size_t offset = 0;
	while ((offset = s.find('<')) != string::npos){
		size_t end = s.find('>', offset);
		s.erase(offset, end + 1 - offset);
	}
	return s;
}
