#include "stdafx.h"
#include "afxmt.h"
#include "IconFile.h"
#include "Log.h"
#include "DataConvert.h"
#include "IconObjectPS2.h"

using namespace std;

CCriticalSection CRITICAL_SECTION_ICON_FILE;

IconFile::IconFile(const std::string& file_path)
{
	Load(file_path);
}

void IconFile::Load(const std::string& file_path)
{

	CRITICAL_SECTION_ICON_FILE.Lock();

	m_bIsOpen = false;
	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan)){
		Log::Add("ACRt@CJ܂ł " + file_path);
		return;
	}

	uint32 version;
	if (file.Read(&version, sizeof(uint32)) != sizeof(uint32) || version != 0x00010000){
		Log::Add("ACRt@Cǂݍ݃G[(o[Wio[) " + file_path);
		return;
	}

	if (file.Read(&m_ModelSectionHeader, sizeof(IconFile::MODEL_SECTION_HEADER)) != sizeof(IconFile::MODEL_SECTION_HEADER)){
		Log::Add("ACRt@Cǂݍ݃G[(fZNVwb_) " + file_path);
		return;
	}

	m_vecVertex.resize(m_ModelSectionHeader.number_of_vertices);
	for (int i = 0; i < (int)m_ModelSectionHeader.number_of_vertices; ++i){
		VERTEX& v = m_vecVertex[i];
		v.coordinate.reserve(m_ModelSectionHeader.number_of_key_shapes);
		for (int j = 0; j < (int)m_ModelSectionHeader.number_of_key_shapes; ++j){
			int16 buffer[4];
			if (file.Read(buffer, sizeof(buffer)) != sizeof(buffer)){
				Log::Add("ACRt@Cǂݍ݃G[(_W) " + file_path);
			}
			v.coordinate.push_back(D3DXVECTOR3(DataConvert::Int16ToFloat(buffer[0]), DataConvert::Int16ToFloat(buffer[1]), DataConvert::Int16ToFloat(buffer[2])));
		}
		int16 normal[4];
		if (file.Read(normal, sizeof(normal)) != sizeof(normal)){
			Log::Add("ACRt@Cǂݍ݃G[() " + file_path);
			return;
		}
		v.normal = D3DXVECTOR3(DataConvert::Int16ToFloat(normal[0]), DataConvert::Int16ToFloat(normal[1]), DataConvert::Int16ToFloat(normal[2]));
		int16 tc[2];
		if (file.Read(&tc, sizeof(tc)) != sizeof(tc)){
			Log::Add("ACRt@Cǂݍ݃G[(eNX`W) " + file_path);
			return;
		}
		v.texture_coordinate = D3DXVECTOR2(DataConvert::Int16ToFloat(tc[0]), DataConvert::Int16ToFloat(tc[1]));
		uint32 color;
		if (file.Read(&color, sizeof(color)) != sizeof(color)){
			Log::Add("ACRt@Cǂݍ݃G[(_F) " + file_path);
			return;
		}
		v.color = color;
	}

	if (file.Read(&m_AnimationSectionHeader, sizeof(m_AnimationSectionHeader)) != sizeof(m_AnimationSectionHeader)){
		Log::Add("ACRt@Cǂݍ݃G[(Aj[VZNVwb_) " + file_path);
		return;
	}

	m_vecAnimationData.resize(m_AnimationSectionHeader.number_of_shapes_to_use);
	for (int i = 0; i < (int)m_AnimationSectionHeader.number_of_shapes_to_use; ++i){
		IconFile::ANIMATION_DATA& ad = m_vecAnimationData[i];
		if (file.Read(&ad.shape_number_of_shape_key, sizeof(ad.shape_number_of_shape_key)) != sizeof(ad.shape_number_of_shape_key)){
			Log::Add("ACRt@Cǂݍ݃G[(Aj[Vsnosk) " + file_path);
			return;
		}

		if (file.Read(&ad.number_of_key_frames, sizeof(ad.number_of_key_frames)) != sizeof(ad.number_of_key_frames)){
			Log::Add("ACRt@Cǂݍ݃G[(Aj[Vnkf) " + file_path);
			return;
		}

		if (ad.number_of_key_frames == 0){
			continue;

		}
		ad.key_data.resize(4 * ad.number_of_key_frames);
		if (file.Read(&ad.key_data[0], 4 * ad.number_of_key_frames * sizeof(int16)) != 4 * ad.number_of_key_frames * sizeof(int16)){
			Log::Add("ACRt@Cǂݍ݃G[(Aj[Vf[^) " + file_path);
			return;
		}
	}

	vector<int16> vec_texture;
	vec_texture.reserve(128 * 128);
	if (!(m_ModelSectionHeader.attributes & 0x4)){
		vec_texture.resize(128 * 128, 0x4210);
	} else if (m_ModelSectionHeader.attributes & 0x8){
		//RLEk
		uint32 file_size;
		if (file.Read(&file_size, sizeof(file_size)) != sizeof(file_size)){
			Log::Add("ACRt@Cǂݍ݃G[(eNX`f[^f[^(RLE)(file_size)) " + file_path);
			return;
		}
		int16 n;
		while (file.Read(&n, sizeof(n)) == sizeof(n)){
			if (n > 0){
				int16 data;
				if (file.Read(&data, sizeof(data)) != sizeof(data)){
					Log::Add("ACRt@Cǂݍ݃G[(eNX`f[^f[^(RLE)(data)) " + file_path);
					return;
				}
				vec_texture.insert(vec_texture.end(), n, data);
			} else if (n < 0){
				n *= -1;
				vector<int16> vec_data(n);
				if (file.Read(&vec_data[0], sizeof(int16) * n) != sizeof(int16) * n){
					Log::Add("ACRt@Cǂݍ݃G[(eNX`f[^f[^(RLE)(vec_data)) " + file_path);
					return;
				}
				vec_texture.insert(vec_texture.end(), vec_data.begin(), vec_data.end());
			} else {
				Log::Add("ACRt@Cǂݍ݃G[(eNX`f[^f[^(RLE)(n)) " + file_path);
				//return;
			}
		}
	} else {
		//ʏ
		vec_texture.resize(2 * 16384);
		if (file.Read(&vec_texture[0], 2 * 16384) != 2 * 16384){
			Log::Add("ACRt@Cǂݍ݃G[(eNX`f[^(raw)) " + file_path);
			return;
		}
	}
	m_vecTextureData.reserve(128 * 128 * 4);
	for (vector<int16>::iterator it = vec_texture.begin(); it != vec_texture.end(); ++it){
		char a = (*it | 0x8000) ? 0xff : 0x00;
		float fc[3];
		fc[0] = (float)((*it & 0x001f) << 3);
		fc[1] = (float)((*it & 0x03e0) >> 2);
		fc[2] = (float)((*it & 0x7c00) >> 7);
		char c[3];
		for (int i = 0; i < 3; ++i){
			float f = fc[i] * (float)(0x100) / (float)(0xf8);
			 c[i] = (char)min(f, 255.5f);
		}
		m_vecTextureData.push_back(c[2]);
		m_vecTextureData.push_back(c[1]);
		m_vecTextureData.push_back(c[0]);
		m_vecTextureData.push_back(a);
	}
	m_bIsOpen = true;

	CRITICAL_SECTION_ICON_FILE.Unlock();
}

void IconFile::SetVertex(int time, void *ptr)const
{
	const ANIMATION_SECTION_HEADER& ah = m_AnimationSectionHeader;
	const vector<ANIMATION_DATA>& ad = m_vecAnimationData;

	const int CYCLE = 1000;
	float f = (float)(time % CYCLE) * (float)ad.size() / (float)CYCLE;
	int frame1 = (int)f;
	int frame2 = frame1 + 1;
	float f1 = (float)frame1;
	float f2 = (float)frame2;
	float r1 = (f2 - f) / (f2 - f1);
	float r2 = 1.0f - r1;
	int animation_size = (int)ad.size();
	if (animation_size == 0){
		++animation_size;
	}
	frame2 %= animation_size;

	IconObjectPS2::VERTEX *vertex = (IconObjectPS2::VERTEX*)ptr;

	for (vector<VERTEX>::const_iterator it = m_vecVertex.begin(); it != m_vecVertex.end(); ++it, ++vertex){
		vertex->p = -(it->coordinate[frame1] * r1 + it->coordinate[frame2] * r2);
		vertex->n = it->normal;
		vertex->c = it->color;
		vertex->t = it->texture_coordinate;
	}
}
