#ifndef ICON_FILE_H
#define ICON_FILE_H

#include <d3dx9math.h>
#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>

typedef unsigned long int uint32;
typedef signed short int int16;

class IconFile
{
public:
	struct MODEL_SECTION_HEADER{
		uint32 number_of_key_shapes;
		uint32 attributes;
		float back_face_clipping_value;
		uint32 number_of_vertices;
	};
	struct VERTEX{
		std::vector<D3DXVECTOR3> coordinate;
		D3DXVECTOR3 normal;
		D3DXVECTOR2 texture_coordinate;
		D3DCOLOR color;
	};

	struct ANIMATION_SECTION_HEADER{
		uint32 number_of_animations;
		uint32 animation_length;
		float animation_speed;
		uint32 offset_in_animation_to_play_from;
		uint32 number_of_shapes_to_use;
	};
	struct ANIMATION_DATA{
		uint32 shape_number_of_shape_key;
		uint32 number_of_key_frames;
		std::vector<int16> key_data;
	};
private:
	MODEL_SECTION_HEADER m_ModelSectionHeader;
	std::vector<VERTEX> m_vecVertex;

	ANIMATION_SECTION_HEADER m_AnimationSectionHeader;
	std::vector<ANIMATION_DATA> m_vecAnimationData;

	std::vector<char> m_vecTextureData;
	bool m_bIsOpen;

	void Load(const std::string& file_path);
	void CalcAnimation();

public:
	IconFile(const std::string& file_path);
	const std::vector<char>& GetTextureData()const{return m_vecTextureData;}
	bool IsOpen()const{return m_bIsOpen;}
	void SetVertex(int time, void *ptr)const;
	int GetVertexCount()const{return (int)m_vecVertex.size();}
};

#endif
