#include "stdafx.h"
#include <d3d9.h>
#include "IconObjectPS1.h"
#include "PS1SaveData.h"
#include "IconTexture.h"

using namespace std;


const IconObjectPS1::VERTEX IconObjectPS1::BILLBOARD[] = {
	{D3DXVECTOR3(-1.0f,  1.0f, 0.0f), D3DXVECTOR2(0.0f, 0.0f)},
	{D3DXVECTOR3( 1.0f,  1.0f, 0.0f), D3DXVECTOR2(1.0f, 0.0f)},
	{D3DXVECTOR3(-1.0f, -1.0f, 0.0f), D3DXVECTOR2(0.0f, 1.0f)},
	{D3DXVECTOR3( 1.0f, -1.0f, 0.0f), D3DXVECTOR2(1.0f, 1.0f)}
};
const DWORD IconObjectPS1::FVF = (D3DFVF_XYZ | D3DFVF_TEX1);
const int TEXTURE_SIZE = 16;
const int ANIMATION_CYCLE = 500;

IconObjectPS1::IconObjectPS1(IDirect3DDevice9* device, const std::string& file_path) :
IconObjectBase(device), m_iTime(0)
{
	PS1SaveData save_data(file_path);
	const vector<vector<char> >& vec_texture = save_data.GetTextureData();
	for (vector<vector<char> >::const_iterator it = vec_texture.begin(); it != vec_texture.end(); ++it){
		IconTexture icon_texture(device, *it, TEXTURE_SIZE, TEXTURE_SIZE);
		m_vecTexture.push_back(icon_texture.GetTexture());
	}
}

IconObjectPS1::~IconObjectPS1()
{
	m_vecTexture.clear();
}

void IconObjectPS1::Update(int time)
{
	m_iTime = time;
}

void IconObjectPS1::Draw()const
{
	m_pDevice->SetRenderState(D3DRS_ZENABLE, FALSE);
	m_pDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	m_pDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	m_pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
	m_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	m_pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	m_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	m_pDevice->SetRenderState(D3DRS_COLORVERTEX, FALSE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_CURRENT);
	m_pDevice->SetTexture(0, m_vecTexture[(m_iTime / ANIMATION_CYCLE) % m_vecTexture.size()].get());

	D3DXMATRIX identity;
	D3DXMatrixIdentity(&identity);
	m_pDevice->SetTransform(D3DTS_WORLD, &identity);
	m_pDevice->SetTransform(D3DTS_VIEW, &identity);
	m_pDevice->SetTransform(D3DTS_PROJECTION, &identity);

	m_pDevice->SetFVF(FVF);

	m_pDevice->BeginScene();
	m_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, BILLBOARD, sizeof(VERTEX));
	m_pDevice->EndScene();
	m_pDevice->Present(NULL, NULL, NULL, NULL);
}
