#include "stdafx.h"
#include "IconObjectPS2.h"
#include "IconFile.h"
#include "IconTexture.h"
#include "IconSys.h"
#include "Utility.h"
#include "VertexBuffer.h"

using namespace std;

const int TEXTURE_SIZE = 128;
const DWORD IconObjectPS2::FVF = D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX1 | D3DFVF_DIFFUSE;

IconObjectPS2::IconObjectPS2(IDirect3DDevice9* device, const std::string& icon_sys_file_path) : IconObjectBase(device), m_bIsOpened(false)
{
	IconSys icon_sys(icon_sys_file_path);
	if (!icon_sys.IsOpen()){
		return;
	}

	const string& icon_file_path = Utility::SplitFileFolder(icon_sys_file_path) + icon_sys.GetIconFileName(0);

	if (!Utility::IsFile(icon_file_path)){
		return;
	}

	m_pIconFile = auto_ptr<IconFile>(new IconFile(icon_file_path));
	IconTexture icon_texture(device, m_pIconFile->GetTextureData(), TEXTURE_SIZE, TEXTURE_SIZE);
	m_pTexture = icon_texture.GetTexture();

	int vertex_count = m_pIconFile->GetVertexCount();
	m_pVertexBuffer = auto_ptr<VertexBuffer>(new VertexBuffer(device, vertex_count * 4 * sizeof(IconObjectPS2::VERTEX)));

	m_bIsOpened = true;
}

IconObjectPS2::~IconObjectPS2()
{
	m_pIconFile.reset();
}

float HEIGHT = 2.25f;
const D3DXVECTOR3 VIEW_EYE( 0.0f, HEIGHT, 7.5f );
const D3DXVECTOR3 VIEW_AT ( 0.0f, HEIGHT, 0.0f );
const D3DXVECTOR3 VIEW_UP ( 0.0f, 1.0f, 0.0f );

void IconObjectPS2::Update(int time)
{
	if (!m_bIsOpened){
		return;
	}

	m_iTime = time;

	int n = m_pIconFile->GetVertexCount();

	void* ptr = m_pVertexBuffer->Lock(n);
	m_pIconFile->SetVertex(m_iTime, ptr);
	m_pVertexBuffer->Unlock();
}

void IconObjectPS2::Draw()const
{
	if (!m_bIsOpened){
		::Sleep(100);
		return;
	}
	m_pDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
	m_pDevice->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);

	m_pDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	m_pDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	m_pDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	m_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	m_pDevice->SetRenderState(D3DRS_COLORVERTEX, TRUE);
	m_pDevice->SetRenderState(D3DRS_NORMALIZENORMALS, TRUE);
	m_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);

	m_pDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE2X);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);

	D3DXMATRIX world;
	//D3DXMatrixIdentity(&world);
	CalcRotateMatrix(m_iTime, world);
	m_pDevice->SetTransform(D3DTS_WORLD, &world);

	D3DXMATRIX view;
	D3DXMatrixLookAtLH(&view, &VIEW_EYE, &VIEW_AT, &VIEW_UP);
	m_pDevice->SetTransform(D3DTS_VIEW, &view);

	D3DXMATRIX proj;
	D3DXMatrixPerspectiveFovLH(&proj, D3DX_PI/4, 1.0f, 1.0f, 100.0f);
	m_pDevice->SetTransform(D3DTS_PROJECTION, &proj);

	m_pDevice->SetTexture(0, m_pTexture.get());

	m_pDevice->SetFVF(FVF);
	m_pDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
	m_pDevice->BeginScene();
	//m_pDevice->DrawPrimitiveUP(D3DPT_TRIANGLELIST, (UINT)(m_vecVertex.size() / 3), &m_vecVertex[0], sizeof(VERTEX));
	m_pVertexBuffer->Draw();
	m_pDevice->EndScene();
	m_pDevice->Present(NULL, NULL, NULL, NULL);
}
