#include "stdafx.h"
#include <d3d9.h>
#include <d3dx9math.h>
#include <afxmt.h>
#include "IconRenderingManager.h"
#include "IconObjectBase.h"
#include "Log.h"
#include "IconObjectPS1.h"
#include "IconObjectPS2.h"
#include "Control.h"
#include "resource.h"
#include "memccppDlg.h"

using namespace std;

IconRenderingManager::IconRenderingManager(Control& control) :
m_bDrawing(false), m_hEventStart(NULL), m_hEventFinish(NULL), m_Control(control)
{
	HWND hwnd = m_hWnd = control.GetDialog().GetIconView().GetSafeHwnd();

	IDirect3D9* d3d = NULL;
	if ((d3d = Direct3DCreate9(D3D_SDK_VERSION)) == NULL){
		Log::Add("IconRenderingManager::IconRenderingManager() : Direct3D̍쐬Ɏs܂");
		return;
	}
	m_pD3D = boost::intrusive_ptr<IDirect3D9>(d3d, false);
	d3d = NULL;

	D3DDISPLAYMODE d3ddm;
	if (FAILED(m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm))){
		Log::Add("IconRenderingManager::IconRenderingManager() : fBXvC[h擾ł܂ł");
		return;
	}

	D3DPRESENT_PARAMETERS d3dpp = {0};
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_FLIP;
	//d3dpp.BackBufferFormat = d3ddm.Format;
	d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
	//d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	d3dpp.hDeviceWindow = m_hWnd;
	d3dpp.EnableAutoDepthStencil = TRUE;
	//d3dpp.BackBufferCount = 3;
	//d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;

	D3DFORMAT iAutoDepthStencilFormat[] = {D3DFMT_D16, D3DFMT_D32, D3DFMT_D24X8, D3DFMT_UNKNOWN};
	for (int i = 0; i < sizeof(iAutoDepthStencilFormat) / sizeof(D3DFORMAT); ++i){
		d3dpp.AutoDepthStencilFormat = iAutoDepthStencilFormat[i];
		if (FAILED(m_pD3D->CheckDepthStencilMatch(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, d3ddm.Format, d3dpp.BackBufferFormat, d3dpp.AutoDepthStencilFormat))){
			continue;
		}
		break;
	}

	IDirect3DDevice9 *device = NULL;
	if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd, D3DCREATE_HARDWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE | D3DCREATE_MULTITHREADED, &d3dpp, &device))){
		if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hwnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE | D3DCREATE_MULTITHREADED, &d3dpp, &device))){
			if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_REF, hwnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING | D3DCREATE_PUREDEVICE | D3DCREATE_MULTITHREADED, &d3dpp, &device))){
				Log::Add("IconRenderingManager::IconRenderingManager() : Direct3DfoCX̍쐬Ɏs܂");
				return;
			}
		}
	}
	m_pDevice = boost::intrusive_ptr<IDirect3DDevice9>(device, false);
	device = NULL;

	m_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	m_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	m_pDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	m_pDevice->SetSamplerState(0, D3DSAMP_ADDRESSU, D3DTADDRESS_CLAMP);
	m_pDevice->SetSamplerState(0, D3DSAMP_ADDRESSV, D3DTADDRESS_CLAMP);

/*

	for (int i = 0; i < 3; ++i){
		m_pDevice->LightEnable(i, TRUE);
	}

	for (int i = 0; i < 3; ++i){
		IconRender::GetDevice()->SetTextureStageState(i, D3DTSS_COLOROP, D3DTOP_ADD);
		IconRender::GetDevice()->SetTextureStageState(i, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
		IconRender::GetDevice()->SetTextureStageState(i, D3DTSS_COLORARG2, D3DTA_CURRENT);
	}
	IconRender::GetDevice()->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_ADD);
	IconRender::GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	IconRender::GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
*/

	m_pCriticalSection = auto_ptr<CCriticalSection>(new CCriticalSection());

	m_hEventStart = CreateEvent(NULL, FALSE, FALSE, "EVENT_ICON_RENDERER_START");
	m_hEventFinish = CreateEvent(NULL, FALSE, FALSE, "EVENT_ICON_RENDERER_FINISH");

	AfxBeginThread(IconRenderingManager::ThreadProc, this);
}

IconRenderingManager::~IconRenderingManager()
{
	m_bDrawing = false;
	SetEvent(m_hEventStart);

	WaitForSingleObject(m_hEventFinish, INFINITE);

	CloseHandle(m_hEventStart);
	m_hEventStart = NULL;

	CloseHandle(m_hEventFinish);
	m_hEventFinish = NULL;

	m_mapIconTable.clear();

	m_pDevice = NULL;
	m_pD3D = NULL;
}

void IconRenderingManager::SetIcon(const std::string& file_path, const std::string& directory_name, bool is_ps2)
{
	if (!m_Control.GetDialog().GetCheckBoxBool(CHECK_BOX_DISPLAY_ICON)){
		return;
	}

	std::map<std::string, boost::shared_ptr<IconObjectBase> >::iterator it = m_mapIconTable.find(directory_name);
	if (it != m_mapIconTable.end()){
		m_pCriticalSection->Lock();
		m_pCurrentIcon = it->second;
		m_pCriticalSection->Unlock();
	} else {
		const boost::shared_ptr<IconObjectBase>& ptr = AddIcon(file_path, directory_name, is_ps2);

		m_pCriticalSection->Lock();
		m_pCurrentIcon = ptr;
		m_pCriticalSection->Unlock();
	}

	m_bDrawing = true;
	SetEvent(m_hEventStart);
}

const boost::shared_ptr<IconObjectBase>& IconRenderingManager::AddIcon(const std::string& file_path, const std::string& directory_name, bool is_ps2)
{
	pair<std::map<std::string, boost::shared_ptr<IconObjectBase> >::iterator, bool> p;
	if (is_ps2){
		p = m_mapIconTable.insert(make_pair(directory_name, boost::shared_ptr<IconObjectPS2>(new IconObjectPS2(m_pDevice.get(), file_path))));
	} else {
		p = m_mapIconTable.insert(make_pair(directory_name, boost::shared_ptr<IconObjectPS1>(new IconObjectPS1(m_pDevice.get(), file_path))));
	}
	return p.first->second;
}

bool IconRenderingManager::IsCached(const std::string& directory_name)const
{
	return m_mapIconTable.find(directory_name) != m_mapIconTable.end();
}

UINT IconRenderingManager::ThreadProc(LPVOID param)
{
	IconRenderingManager *icon_renderer = (IconRenderingManager*)param;
	icon_renderer->Loop();
	return 0;
}

void IconRenderingManager::Loop()
{
	WaitForSingleObject(m_hEventStart, INFINITE);

	while (m_bDrawing){
		int time = (int)::GetTickCount();

		if (m_Control.GetDialog().GetCheckBoxBool(CHECK_BOX_DISPLAY_ICON)){
			m_pCriticalSection->Lock();

			m_pCurrentIcon->Update(time);
			m_pCurrentIcon->Draw();

			m_pCriticalSection->Unlock();
			::Sleep(0);
		} else {
			::Sleep(100);
		}
	}

	SetEvent(m_hEventFinish);
}
