#ifndef ICON_RENDERING_MANAGER_H
#define ICON_RENDERING_MANAGER_H

struct IDirect3D9;
struct IDirect3DDevice9;
struct IDirect3DTexture9;
class IconObjectBase;
class CCriticalSection;

#include <map>
#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/intrusive_ptr.hpp>

class Control;

class IconRenderingManager
{
private:
	boost::intrusive_ptr<IDirect3D9> m_pD3D;
	boost::intrusive_ptr<IDirect3DDevice9> m_pDevice;
	std::map<std::string, boost::shared_ptr<IconObjectBase> > m_mapIconTable;
	boost::shared_ptr<IconObjectBase> m_pCurrentIcon;
	std::auto_ptr<CCriticalSection> m_pCriticalSection;
	bool m_bDrawing;
	HWND m_hWnd;
	HANDLE m_hEventStart;
	HANDLE m_hEventFinish;
	Control& m_Control;

	static UINT ThreadProc(LPVOID param);
	void Loop();
public:
	IconRenderingManager(Control& conrtol);
	virtual ~IconRenderingManager();
	void SetIcon(const std::string& file_path, const std::string& directory_name, bool is_ps2);
	const boost::shared_ptr<IconObjectBase>& AddIcon(const std::string& file_path, const std::string& directory_name, bool is_ps2);
	bool IsCached(const std::string& directory_name)const;
};

#endif
