#include "stdafx.h"
#include "IconSys.h"
#include "Log.h"
#include "DataConvert.h"

using namespace std;

struct FILE_STRUCT{
	char magic[4];
	char reserve0[2];
	short second_line_offset;
	char reserve1[4];
	long background_transparency;
	long background_color[4][4];
	float light_direction[3][4];
	float light_color[4][4];
	char title[68];
	char icon_file_name[3][64];
	char reserve2[60];
};

const char *MagicNumber = "PS2D";
const int iBufferSize = 1024;

IconSys::IconSys(const std::string& file_path)
{
	m_bIsOpen = false;
	CFile file;
	CFileException e;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone, &e)){
		Log::Add("t@C̓ǂݍ݂Ɏs܂ IconSys::IconSys " + file_path);
		TCHAR buffer[iBufferSize + 1];
		e.GetErrorMessage(buffer, iBufferSize);
		Log::Add(buffer);
		return;
	}
	FILE_STRUCT file_struct;
	if (file.Read(&file_struct, sizeof(FILE_STRUCT)) != sizeof(FILE_STRUCT)){
		Log::Add("t@C̓ǂݍ݂Ɏs܂ IconSys::IconSys " + file_path);
		return;
	}

	for (int i = 0; i < 4; ++i){
		if (file_struct.magic[i] != MagicNumber[i]){
			Log::Add("t@C̓ǂݍ݂Ɏs܂ IconSys::IconSys " + file_path);
			return;
		}
	}
	float transparency = DataConvert::CharToFloat((char)file_struct.background_transparency);
	for (int i = 0; i < 4; ++i){
		float r = (float)file_struct.background_color[i][0] / 256.0f;
		float g = (float)file_struct.background_color[i][1] / 256.0f;
		float b = (float)file_struct.background_color[i][2] / 256.0f;
		m_BackgroundColor[i] = (DWORD)D3DXCOLOR(r, g, b, transparency);
	}
	for (int i = 0; i < 3; ++i){
		float x = file_struct.light_direction[i][0];
		float y = file_struct.light_direction[i][1];
		float z = file_struct.light_direction[i][2];
		m_LightDirection[i] = D3DXVECTOR3(x, y, z) * -1.0f;
	}
	for (int i = 0; i < 4; ++i){
		float r = file_struct.light_color[i][0];
		float g = file_struct.light_color[i][1];
		float b = file_struct.light_color[i][2];
		float a = 1.0f;
		m_LightColor[i] = D3DXCOLOR(r, g, b, a);
	}
	m_iSecondLineOffset = file_struct.second_line_offset;
	m_strTitle = file_struct.title;
	if ((int)m_strTitle.size() > m_iSecondLineOffset){
		m_strTitle.insert(m_iSecondLineOffset, " ");
	}
	for (int i = 0; i < 3; ++i){
		m_strIconFileName[i] = file_struct.icon_file_name[i];
	}
	m_bIsOpen = true;
}
