#include "stdafx.h"
#include <ddraw.h>
#include <d3d9.h>
#include <d3dx9tex.h>
#include "IconTexture.h"

using namespace std;

IconTexture::IconTexture(IDirect3DDevice9 *device) : m_pDevice(device), m_iWidth(0), m_iHeight(0)
{
}

IconTexture::IconTexture(IDirect3DDevice9 *device, const std::string& file_path) : m_pDevice(device)
{
	Load(file_path);
}

const char strMagic[] = "DDS ";

IconTexture::IconTexture(IDirect3DDevice9 *device, const std::vector<char>& pixel, int height, int width) :
m_pDevice(device), m_iWidth(width), m_iHeight(height)
{
	int file_size = 4 + sizeof(DDSURFACEDESC2) + height * width * 4;
	vector<char> buffer(file_size);

	CopyMemory(&buffer[0], strMagic, sizeof(strMagic));

	DDSURFACEDESC2 *header = (DDSURFACEDESC2 *)&buffer[4];
	header->dwSize = sizeof(DDSURFACEDESC2);
	header->dwFlags = DDSD_CAPS | DDSD_PIXELFORMAT | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PITCH;
	header->dwHeight = height;
	header->dwWidth = width;
	header->lPitch = height * 4;
	header->ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	header->ddpfPixelFormat.dwFlags = DDPF_RGB;
	header->ddpfPixelFormat.dwRGBBitCount = 32;
	header->ddpfPixelFormat.dwRBitMask = 0x00ff0000;
	header->ddpfPixelFormat.dwGBitMask = 0x0000ff00;
	header->ddpfPixelFormat.dwBBitMask = 0x000000ff;
	header->ddpfPixelFormat.dwRGBAlphaBitMask = 0xff000000;
	header->ddsCaps.dwCaps = DDSCAPS_TEXTURE;
	CopyMemory(&buffer[128], &pixel[0], height * width * 4);

	IDirect3DTexture9 *texture = NULL;
	D3DXCreateTextureFromFileInMemory(m_pDevice, &buffer[0], file_size, &texture);
	m_pTexture = boost::intrusive_ptr<IDirect3DTexture9>(texture, false);
	texture = NULL;

	//static int counter = 0;
	//D3DXSaveTextureToFile(string("texture" + Utility::ToString(counter++, 2) + ".dds").c_str(), D3DXIFF_DDS, m_pTexture.get(), NULL);
}

IconTexture::~IconTexture()
{
	m_iHeight = m_iWidth = 0;
}

void IconTexture::Load(const std::string& file_path)
{
	IDirect3DTexture9 *texture = NULL;
	D3DXCreateTextureFromFile(m_pDevice, file_path.c_str(), &texture);
	m_pTexture = boost::intrusive_ptr<IDirect3DTexture9>(texture, false);
	texture = NULL;

	IDirect3DSurface9 *surface = 0;
	m_pTexture->GetSurfaceLevel(0, &surface);

	D3DSURFACE_DESC desc;
	surface->GetDesc(&desc);
	m_iHeight = (int)desc.Height;
	m_iWidth = (int)desc.Width;

	SAFE_RELEASE(surface);
}

void IconTexture::Save(const std::string& file_path)const
{
	D3DXSaveTextureToFile(file_path.c_str(), D3DXIFF_BMP, m_pTexture.get(), NULL);
}
