#include "stdafx.h"
#include "PS1SaveFileMcr.h"
#include "PS1SaveData.h"

using namespace std;

PS1SaveFileMcr::HEADER::HEADER()
{
	ZeroMemory(this, sizeof(HEADER));
	magic[0] = 'M';
	magic[1] = 'C';
	xor[0] = CalcXor((const char*)this);
}

PS1SaveFileMcr::SOFT_HEADER::SOFT_HEADER()
{
	ZeroMemory(this, sizeof(SOFT_HEADER));
}

PS1SaveFileMcr::SOFT_HEADER::SOFT_HEADER(int flag, int block, int link, const std::string& id)
{
	ZeroMemory(this, sizeof(SOFT_HEADER));
	this->flag[0] = (char)flag;
	*(int*)(&this->block) = (int)block;
	*(short*)(&this->link) = (short)link;
	lstrcpy(this->name, id.c_str());
	xor[0] = CalcXor((const char*)this);
}

PS1SaveFileMcr::PS1SaveFileMcr() : m_iUsedBlock(0)
{
}

PS1SaveFileMcr::~PS1SaveFileMcr()
{
}

char PS1SaveFileMcr::CalcXor(const char *ptr)
{
	char res = 0;
	for (int i = 0; i < 127; ++i){
		res ^= ptr[i];
	}
	return res;
}

bool PS1SaveFileMcr::LoadRaw(const std::string& file_path)
{
	int file_size;
	if (!Utility::IsFile(file_path) || (file_size = (int)Utility::GetFileSize(file_path)) % 8192 != 0 || !PS1SaveData(file_path).IsOpen()){
		Log::Add("PS1SaveFileMcr::LoadRaw() : PS1Z[uf[^łȂ\܂ : " + file_path);
		return false;
	}

	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary | CFile::osSequentialScan)){
		Log::Add("PS1SaveFileMcr::LoadRaw() : t@CJ܂ł : " + file_path);
		return false;
	}

	const string id = Utility::SplitFileName(file_path);
	vector<char> buffer(file_size);
	if (file.Read(&buffer[0], file_size) != file_size){
		Log::Add("PS1SaveFileMcr::LoadRaw() : t@C̓ǂݍ݂Ɏs܂ : " + file_path);
		return false;
	}

	m_vecData.push_back(make_pair(id, buffer));

	return true;
}

bool PS1SaveFileMcr::SaveRaw(const std::string& directory_path)const
{
	for (vector<pair<string, vector<char> > >::const_iterator it = m_vecData.begin(); it != m_vecData.end(); ++it){
		const string file_path = directory_path + "\\" + it->first;
		CFile file;
		if (!file.Open(file_path.c_str(), CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone | CFile::typeBinary)){
			Log::Add("PS1SaveFileMcr::SaveRaw() : t@CJ܂ł : " + file_path);
			return false;
		}

		file.Write(&(it->second[0]), (UINT)it->second.size());
	}
	return true;
}

bool PS1SaveFileMcr::Load(const std::string& file_path)
{
	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary)){
		Log::Add("PS1SaveFileMcr::Load() : t@CJ܂ł : " + file_path);
		return false;
	}

	HEADER header;
	file.Read(&header, sizeof(header));

	SOFT_HEADER soft_header[15];
	file.Read(soft_header, sizeof(soft_header));

	char data[15][0x2000];
	file.Seek(0x2000, CFile::begin);
	file.Read(data, sizeof(data));

	int finish_flag[15];
	ZeroMemory(finish_flag, sizeof(finish_flag));

	for (int i = 0; i < 15; ++i){
		if (finish_flag[i]){
			continue;
		}
		finish_flag[i] = true;

		if (soft_header[i].flag[0] == 0xa0){
			continue;
		}

		int index = i;
		vector<char> v;
		v.reserve(16 * 0x2000);

		do {
			finish_flag[index] = true;
			v.insert(v.end(), data[index], data[index] + 0x2000);
			const string name = soft_header[index].name;
			index = *(short*)soft_header[index].link;
			m_vecData.push_back(make_pair(name, v));
		} while (index >= 0);

	}
	return true;
}

bool PS1SaveFileMcr::Save(const std::string& file_path)const
{
	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone | CFile::typeBinary)){
		Log::Add("PS1SaveFileMcr::Save() : t@CJ܂ł : " + file_path);
		return false;
	}

	HEADER header;
	file.Write(&header, sizeof(header));

	int block_index = 0;
	for (vector<pair<string, vector<char> > >::const_iterator it = m_vecData.begin(); it != m_vecData.end(); ++it){
		int use_block = (int)it->second.size() / 0x2000;

		for (int i = 0; i < use_block; ++i){
			int flag;
			if (i == 0){
				flag = 0x51;
			} else  if (i == use_block - 1){
				flag = 0x53;
			} else {
				flag = 0x52;
			}

			int link;
			if (flag == 0x53){
				link = 0xffff;
			} else {
				link = block_index + 1;
			}

			SOFT_HEADER soft_header(flag, use_block * 0x2000, link, it->first);
			file.Write(&soft_header, sizeof(soft_header));
			++block_index;
		}
	}

	if (block_index > 15){
		Log::Add("PS1SaveFileMcr::Save() : KvubN܂");
		return false;
	}

	for (; block_index < 15; ++block_index){
		SOFT_HEADER soft_header(0xa0, 0, 0, "");
		file.Write(&soft_header, sizeof(soft_header));
	}

	file.Seek(0x2000, CFile::begin);

	vector<char> v;
	v.reserve(16 * 0x2000);
	for (vector<pair<string, vector<char> > >::const_iterator it = m_vecData.begin(); it != m_vecData.end(); ++it){
		v.insert(v.end(), it->second.begin(), it->second.end());
	}
	v.resize(0x2000 * 15);
	file.Write(&v[0], 0x2000 * 15);
	return true;
}
