#include "stdafx.h"
#include "SaveFileFolder.h"
#include "Utility.h"
#include "Log.h"

using namespace std;

const int iReserveFileCount = 16;

bool SaveFileFolder::Save(const std::string& folder_path)const
{
	if (m_mapFileData.empty()){
		return false;
	}
	if (!Utility::IsFolder(folder_path)){
		return false;
	}
	for (map<string, FOLDER_DATA>::const_iterator it = m_mapFileData.begin(); it != m_mapFileData.end(); ++it){
		const FOLDER_DATA& folder_data = it->second;
		const string folder_path2 = folder_path + "\\" + it->first;
		if (!Utility::IsFolder(folder_path2)){
			MoveFile(folder_path2.c_str(), Utility::GenerateBackupFileName(folder_path2).c_str());
		}
		CreateDirectory(folder_path2.c_str(), NULL);

		for (FOLDER_DATA::const_iterator itf = folder_data.begin(); itf != folder_data.end(); ++itf){
			const FILE_DATA& file_data = **itf;
			const string file_path = folder_path2 + "\\" + file_data.file_name;

			CFile file;
			CFileException e;
			if (!file.Open(file_path.c_str(), CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone, &e)){
				Log::Add("t@C̍쐬Ɏs܂ : " + file_path);
				TCHAR buffer[1024];
				e.GetErrorMessage(buffer, 1024);
				Log::Add(buffer);
				continue;
			}
			if (!file_data.body.empty()){
				//t@CTCY0̏ꍇ炵
				file.Write(&(file_data.body[0]), (UINT)file_data.body.size());
			}
			file.Close();

			CFileStatus file_status;
			CFile::GetStatus(file_path.c_str(), file_status);
			file_status.m_ctime = file_data.status.m_ctime;
			file_status.m_mtime = file_data.status.m_mtime;
			file_status.m_atime = file_data.status.m_atime;
			file_status.m_attribute = file_data.status.m_attribute;
			CFile::SetStatus(file_path.c_str(), file_status);
		}
	}
	return true;
}

bool SaveFileFolder::Load(const std::string& file_path)
{
	if (!Utility::IsFolder(file_path)){
		return false;
	}
	const string folder_name = file_path.substr(file_path.find_last_of('\\') + 1);
	m_mapFileData.clear();
	SaveFileBase::FOLDER_DATA& folder = m_mapFileData[folder_name];

	CFileFind finder;
	BOOL working = finder.FindFile((file_path + "\\*.*").c_str());
	while (working){
		working = finder.FindNextFile();
		if (finder.IsDots() || finder.IsDirectory()){
			continue;
		}

		CFile file((LPCTSTR)finder.GetFilePath(), CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan);
		boost::shared_ptr<FILE_DATA> file_data(new FILE_DATA);
		file_data->file_name = (LPCTSTR)finder.GetFileName();
		file.GetStatus(file_data->status);
		file_data->body.resize((size_t)file.GetLength());
		if (file.GetLength()){
			file.Read(&file_data->body[0], (UINT)file.GetLength());
		}
		file.Close();
		folder.push_back(file_data);
	}
	sort(folder.begin(), folder.end(), SaveFileBase::SortFile());
	return true;
}
