#include "stdafx.h"
#include "SaveFileNPO.h"
#include "Log.h"
#include "Utility.h"

using namespace std;

struct NPO_HEADER
{
	char identify[8];
	int file_count;
	int offset_icon_sys;
	int offset_icon_file;
	char reserve[4];
};

struct NPO_FILE_HEADER
{
	char name[48];
	int file_size;
	char reserve0[4];
};

bool SaveFileNPO::Save(const std::string& folder_path)const
{
	for (map<string, FOLDER_DATA>::const_iterator it_folder = m_mapFileData.begin(); it_folder != m_mapFileData.end(); ++it_folder){
		const string& folder_name = it_folder->first;
		const SaveFileBase::FOLDER_DATA& folder_data = it_folder->second;

		const string file_path = folder_path + "\\" + folder_name + ".npo";
		CFile file;
		CFileException pError;
		if (!file.Open(file_path.c_str(), CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone, &pError)){
			Log::Add("SaveFileNPO::Save() t@CJ܂ł " + file_path);
			char buffer[512];
			pError.GetErrorMessage(buffer, 512);
			Log::Add(buffer);
			return false;
		}
		NPO_HEADER npo_header = {0};
		strcpy_s(npo_header.identify, sizeof(npo_header.identify), "nPort");

		npo_header.file_count = (int)folder_data.size() + 1;
		int offset_icon_sys = 24;
		string icon_file_name;

		for (FOLDER_DATA::const_iterator it_file = folder_data.begin(); it_file != folder_data.end(); ++it_file){
			const FILE_DATA& file_data = **it_file;
			offset_icon_sys += sizeof(NPO_FILE_HEADER);
			if (file_data.file_name == "icon.sys"){
				char buffer[128];
				ZeroMemory(buffer, sizeof(buffer));
				strncpy_s(buffer, sizeof(buffer), &(file_data.body[0x104]), 64);
				icon_file_name = buffer;
				break;
			}
			offset_icon_sys += (int)file_data.body.size();
		}
		npo_header.offset_icon_sys = offset_icon_sys;

		int offset_icon_file = 24;
		for (FOLDER_DATA::const_iterator it_file = folder_data.begin(); it_file != folder_data.end(); ++it_file){
			const FILE_DATA& file_data = **it_file;
			offset_icon_file += sizeof(NPO_FILE_HEADER);
			if (file_data.file_name == icon_file_name){
				break;
			}
			offset_icon_file += (int)file_data.body.size();
		}
		npo_header.offset_icon_file = offset_icon_file;

		file.Write(&npo_header, sizeof(npo_header));
		for (FOLDER_DATA::const_iterator it_file = folder_data.begin(); it_file != folder_data.end(); ++it_file){
			const FILE_DATA& file_data = **it_file;
			NPO_FILE_HEADER npo_file_header = {0};
			strcpy_s(npo_file_header.name, sizeof(npo_file_header.name), file_data.file_name.c_str());
			npo_file_header.file_size = (int)file_data.body.size();
			file.Write(&npo_file_header, sizeof(npo_file_header));
			file.Write(&file_data.body[0], (int)file_data.body.size());
		}
		char empty_buffer[56];
		ZeroMemory(empty_buffer, sizeof(empty_buffer));
		file.Write(empty_buffer, sizeof(empty_buffer));
	}
	return true;
}

bool SaveFileNPO::Load(const std::string& file_path)
{
	CFile file;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan)){
		Log::Add("SaveFileNPO::Load() t@CJ܂ł");
		return false;
	}
	const string file_name = Utility::SplitFileTitle(file_path);
	m_mapFileData.clear();
	FOLDER_DATA& folder_data = m_mapFileData[file_name];

	NPO_HEADER npo_header;
	if (file.Read(&npo_header, sizeof(npo_header)) != sizeof(npo_header)){
		Log::Add("SaveFileNPO::Load() wb_ǂ߂܂ł");
		return false;
	}
	--npo_header.file_count;
	folder_data.reserve(npo_header.file_count);

	for (int i = 0; i < npo_header.file_count; ++i){
		boost::shared_ptr<FILE_DATA> file_data(new FILE_DATA());

		NPO_FILE_HEADER npo_file_header;
		if (file.Read(&npo_file_header, sizeof(npo_file_header)) != sizeof(npo_file_header)){
			Log::Add("SaveFileNPO::Load() t@Cwb_ǂ߂܂ł");
			return false;
		}
		file_data->body.resize(npo_file_header.file_size);
		if (file.Read(&file_data->body[0], npo_file_header.file_size) != npo_file_header.file_size){
			Log::Add("SaveFileNPO::Load() t@Cǂ߂܂ł");
			return false;
		}
		char file_name_buffer[64];
		strncpy_s(file_name_buffer, sizeof(file_name_buffer), npo_file_header.name, 63);
		file_data->file_name = file_name_buffer;
		ZeroMemory(&file_data->status, sizeof(file_data->status));
		file_data->status.m_attribute = CFile::normal;
		strcpy_s(file_data->status.m_szFullName, sizeof(file_data->status.m_szFullName), file_name_buffer);
		file_data->status.m_atime = file_data->status.m_ctime = file_data->status.m_mtime = CTime::GetCurrentTime();
		file_data->status.m_ctime -= CTimeSpan(0, 0, 0, file_data->status.m_ctime.GetSecond());
		file_data->status.m_ctime += CTimeSpan(0, 0, 0, i + 1);
		file_data->status.m_size = (int)file_data->body.size();
	}
	return true;
}
