#include "stdafx.h"
#include <algorithm>
#include <functional>
#include "SaveFilePSU.h"
#include "Log.h"
#include "Utility.h"

using namespace std;

const int iClusterSize = 0x400;

struct PSU_TIME{
	unsigned char second;
	unsigned char minute;
	unsigned char hour;
	unsigned char day;
	unsigned char month;
	unsigned char year[2];
	PSU_TIME(){
		ZeroMemory(this, sizeof(PSU_TIME));
	}
	PSU_TIME(const CTime& time) : second((char)time.GetSecond()), minute((char)time.GetMinute()), hour((char)time.GetHour()), day((char)time.GetDay()), month((char)time.GetMonth()){
		year[0] = ((char)(time.GetYear() % 0x100));
		year[1] = ((char)(time.GetYear() / 0x100));
	}
	CTime GetCTime()const{
		int y = (int)year[0] + (int)year[1] * 0x100;
		if (y == 0){
			y = 2000;
		}
		return CTime(y, std::max(1, (int)month), std::max(1, (int)day), hour, minute, second);
	}

};

struct PSU_HEADER{
	unsigned char file_status[2];
	unsigned char zero0[2];
	long file_size;
	unsigned char zero1[1];
	PSU_TIME time_create;
/*
	char csecond;
	char cminute;
	char chour;
	char cday;
	char cmonth;
	short cyear;
*/
	unsigned char zero2[4];
	unsigned char zero3[5];
	PSU_TIME time_update;
/*
	char usecond;
	char uminute;
	char uhour;
	char uday;
	char umonth;
	short uyear;
*/

	char zero4[0x20];
	char name[0x1c0];

	PSU_HEADER(){
//#ifdef _DEBUG
//		Log::Add("vO : PSU_HEADERTCY = " + Utility::ToString(sizeof(PSU_HEADER)));
//		Log::Add("vO : PSU_TIME  TCY = " + Utility::ToString(sizeof(PSU_TIME)));
//#endif
		ZeroMemory(this, sizeof(PSU_HEADER));
	}
};

const std::string SaveFilePSU::MakeFileTitle(const std::string& save_data_title)const
{
	string res = Utility::ToHalf(save_data_title);
	if (res[0] == ' '){
		res.erase(0, res.find_first_not_of(' '));
	}

	if (res[res.size() - 1]){
		res.erase(res.find_last_not_of(' ') + 1, string::npos);
	}

	size_t index_space;
	while ((index_space = res.find("  ")) != string::npos){
		res.erase(index_space, 1);
	}

	return Utility::ToFull(res);
}

bool SaveFilePSU::Save(const std::string& folder_path)const
{
	if (m_mapFileData.empty()){
		return false;
	}
	if (!Utility::IsFolder(folder_path)){
		return false;
	}
	for (map<string, SaveFileBase::FOLDER_DATA>::const_iterator it = m_mapFileData.begin(); it != m_mapFileData.end(); ++it){
		const string& folder_name = it->first;
		const SaveFileBase::FOLDER_DATA& folder_data = it->second;

		string file_title = "j̧";
		for (FOLDER_DATA::const_iterator itf = folder_data.begin(); itf != folder_data.end(); ++itf){
			const FILE_DATA& file_data = **itf;
			if (file_data.file_name == "icon.sys"){
				const string save_data_title = string(&(file_data.body[192]), &(file_data.body[192 + 64]));
				file_title = MakeFileTitle(save_data_title);
				break;
			}
		}

		const string file_path = folder_path + "\\" + Utility::ToFull(file_title) + ".psu";
		if (Utility::IsFile(file_path)){
			MoveFile(file_path.c_str(), Utility::GenerateBackupFileName(file_path).c_str());
		}

		CFile file;
		CFileException e;
		if (!file.Open(file_path.c_str(), CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone, &e)){
			Log::Add("t@C̍쐬Ɏs܂ SaveFilePSU::Save()");
			Log::Add(file_path);
			TCHAR buffer[1024];
			e.GetErrorMessage(buffer, 1024);
			Log::Add(buffer);
		}

		CFileStatus file_status;
		file.GetStatus(file_status);

		PSU_HEADER header;
		lstrcpy(header.name, folder_name.c_str());
		header.file_status[0] = 0x27;
		header.file_status[1] = 0x84;
		header.file_size = (int)(folder_data.size() + 2);
		header.time_create = PSU_TIME(file_status.m_ctime);
		header.time_update = PSU_TIME(file_status.m_mtime);
		file.Write(&header, sizeof(PSU_HEADER));

		PSU_HEADER header1;
		lstrcpy(header1.name, ".");
		header1.file_status[0] = 0x27;
		header1.file_status[1] = 0x84;
		header1.time_create = PSU_TIME(file_status.m_ctime);
		header1.time_update = PSU_TIME(file_status.m_mtime);
		file.Write(&header1, sizeof(PSU_HEADER));

		PSU_HEADER header2;
		lstrcpy(header2.name, "..");
		header2.file_status[0] = 0x27;
		header2.file_status[1] = 0x84;
		header2.time_create = PSU_TIME(file_status.m_ctime);
		header2.time_update = PSU_TIME(file_status.m_mtime);
		file.Write(&header2, sizeof(PSU_HEADER));

		for (FOLDER_DATA::const_iterator it_file = folder_data.begin(); it_file != folder_data.end(); ++it_file){
			const FILE_DATA& file_data = **it_file;
			PSU_HEADER file_header;
			lstrcpy(file_header.name, file_data.file_name.c_str());
			file_header.file_size = (long)file_data.body.size();
			file_header.file_status[0] = 0x97;
			file_header.file_status[1] = 0x84;
			file_header.time_create = PSU_TIME(file_data.status.m_ctime);
			file_header.time_update = PSU_TIME(file_data.status.m_mtime);
			file.Write(&file_header, sizeof(PSU_HEADER));

			int file_size = (int)file_data.body.size();
			if (!file_size){
				continue;
			}
			int file_size_cluster = (((file_size - 1) / iClusterSize) + 1) * iClusterSize;
			auto_ptr<char> buffer(new char[file_size_cluster]);
			ZeroMemory(buffer.get(), file_size_cluster);
			memcpy(buffer.get(), &(file_data.body[0]), file_data.body.size());
			file.Write(buffer.get(), file_size_cluster);
		}

		file.Close();
	}
	return true;
}

bool SaveFilePSU::Load(const std::string& file_path)
{
	if (!Utility::IsFile(file_path)){
		return false;
	}

	CFile file;
	CFileException e;
	if (!file.Open(file_path.c_str(), CFile::modeRead | CFile::shareDenyNone | CFile::osSequentialScan, &e)){
		Log::Add("t@CJ܂ł SaveFilePsu::Load()");
		Log::Add(file_path);
		TCHAR buffer[1024];
		e.GetErrorMessage(buffer, 1024);
		Log::Add(buffer);
		return false;
	}

	int file_count = -1;
	PSU_HEADER file_header;
	file.Read(&file_header, sizeof(PSU_HEADER));
	file_count = (int)file_header.file_size;
	const std::string folder_name = file_header.name;

	m_mapFileData.clear();
	FOLDER_DATA& folder_data = m_mapFileData[folder_name];

	for (int i = 0; i < file_count; ++i){
		PSU_HEADER header;
		file.Read(&header, sizeof(PSU_HEADER));
		if (header.file_status[0] == 0x27 && header.file_status[1] == 0x84){
			continue;
		}

		int file_size = header.file_size;
		int file_size_cluster = (((file_size - 1) / iClusterSize) + 1) * iClusterSize;
		if (file_size == 0){
			file_size_cluster = 0;
		}

		auto_ptr<char> buffer(new char[file_size_cluster]);

		if (file_size_cluster){
			memset(buffer.get(), 0xff, file_size_cluster);
			file.Read(buffer.get(), file_size_cluster);
		}
		boost::shared_ptr<FILE_DATA> file_data(new FILE_DATA());

		file_data->file_name = header.name;
		file_data->body = vector<char>(buffer.get(), buffer.get() + file_size);
		ZeroMemory(&(file_data->status), sizeof(CFileStatus));
		file_data->status.m_ctime = header.time_create.GetCTime();
		file_data->status.m_atime = header.time_create.GetCTime();
		file_data->status.m_mtime = header.time_update.GetCTime();
		file_data->status.m_size = (LONG)file_size;
		file_data->status.m_attribute = CFile::normal;
		folder_data.push_back(file_data);
	}
	sort(folder_data.begin(), folder_data.end(), SaveFileBase::SortFile());
	return true;
}
