#include "stdafx.h"
#include "VertexBuffer.h"
#include "Log.h"
#include "IconObjectPS2.h"

using namespace std;

VertexBuffer::VertexBuffer(IDirect3DDevice9 *device, int buffer_size) : m_pDevice(device), m_iOffsetBegin(0), m_iOffsetEnd(0), m_iBufferSize(buffer_size)
{
	IDirect3DVertexBuffer9 *vertex_buffer = NULL;
	if (FAILED(m_pDevice->CreateVertexBuffer(buffer_size, D3DUSAGE_DYNAMIC, IconObjectPS2::FVF, D3DPOOL_DEFAULT, &vertex_buffer, NULL))){
		Log::Add("VertexBuffer::VertexBuffer() : o[eNXobt@̍쐬Ɏs܂");
		SAFE_RELEASE(vertex_buffer);
		return;
	}
	m_pVertexBuffer = boost::intrusive_ptr<IDirect3DVertexBuffer9>(vertex_buffer, false);
	vertex_buffer = NULL;
}

VertexBuffer::~VertexBuffer()
{
	m_pVertexBuffer = NULL;
}

void* VertexBuffer::Lock(int vertex_count)
{
	int lock_size = sizeof(IconObjectPS2::VERTEX) * vertex_count;

	m_iOffsetBegin = m_iOffsetEnd;
	DWORD lock_flag = D3DLOCK_NOOVERWRITE;
	if (m_iOffsetEnd + lock_size > m_iBufferSize){
		lock_flag = D3DLOCK_DISCARD;
		m_iOffsetBegin = m_iOffsetEnd = 0;
	}

	void* bytes = 0;
	if (FAILED(m_pVertexBuffer->Lock(m_iOffsetBegin, lock_size, (void**)&bytes, lock_flag))){
		Log::Add("VertexBuffer::Lock() : o[eNXobt@̃bNɎs܂");
		return NULL;
	}
	m_iOffsetEnd = m_iOffsetBegin + lock_size;
	return bytes;
}

void VertexBuffer::Unlock()
{
	m_pVertexBuffer->Unlock();
}

void VertexBuffer::Draw()const
{
	int vertex_size = sizeof(IconObjectPS2::VERTEX);
	m_pDevice->SetStreamSource(0, m_pVertexBuffer.get(), m_iOffsetBegin, vertex_size);
	m_pDevice->SetFVF(IconObjectPS2::FVF);
	int vertex_count = (m_iOffsetEnd - m_iOffsetBegin) / vertex_size;
	int primitive_count = vertex_count / 3;
	m_pDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, primitive_count);
}
